
# The Personis system is copyright 2000-2012 University of Sydney
#       Bob.Kummerfeld@Sydney.edu.au

# This file is part of Personis.

# Personis is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Personis is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Personis.  If not, see <http://www.gnu.org/licenses/>.

import httplib2, types, cPickle
import json
import cherrypy, oauth2client
#from ..client import Connection

def do_call(fun, args, connection):
	if (not connection.valid()):
		raise SystemError('Need http or modelserver and credentials')
	args["version"] = "11.2"
	args_json = json.dumps(args)+'\n'

	http = connection.get_http()

	resp, content = http.request(connection.uri+fun, "POST", body=args_json)

	try:
		result = json.loads(content)
	except:
		print "json loads failed!"
		print "<<%s>>" % (content)
		raise ValueError("json loads failed")
	# dirty kludge to get around unicode
	for k,v in result.items():
		if type(v) == type(''):
			result[k] = str(v)
		if type(k) == type(''):
			del result[k]
			result[str(k)] = v
	## Unpack the error, and if it is an exception throw it.
	if type(result) == dict and 'result' in result:
		if result["result"] == "error":
			print result
			# We have returned with an error, so throw it as an exception.
			if 'pythonPickel' in result:
				raise cPickle.loads(result["pythonPickel"])
			elif len(result["val"]) == 3:
				raise cPickle.loads(str(result["val"][2]))
			else:
				raise Exception(str(result["val"]))
		else:
			# Unwrap the result, and return as normal. 
			result = result["val"]
		return result



#!/usr/bin/env python2.4

# NOTE: this test can only be run locally since
# it adds a new resolver.
# - need to dynamically import resolvers on server side

print("""
==============================================================
Test of adding new resolver function
this test can only be run locally since it adds a new resolver
see Base tests
==============================================================
""")
import sys; sys.exit(0)





#!/usr/bin/env python2.4

# NOTE: this test can only be run locally since
# it adds a new resolver.
import Personis_base
from Personis_util import printcomplist

# add some evidence to some components and then use example1 to see it

um = Personis_base.Access(model="Alice", modeldir="Tests/Models", user="alice", password='secret')
# create a piece of evidence with Alice as value
ev = Personis_base.Evidence(evidence_type="explicit", value="Alice")
# tell this as user Alice's first name
um.tell(context=["Personal"], componentid='firstname', evidence=ev)
ev = Personis_base.Evidence(evidence_type="explicit", value="Smith")
um.tell(context=["Personal"], componentid='lastname', evidence=ev)

print("===================================================================")
print("Now add a new resolver to um")

def myresolver(model=None, component=None, context=None, resolver_args=None):
	"""     new resolver function 
	"""
	print("new resolver called with ", repr(component))
	if resolver_args == None:
		ev_filter = None
	else:
		ev_filter = resolver_args.get('evidence_filter')
	component.evidencelist = component.filterevidence(model=model, context=context, resolver_args=resolver_args)
	if len(component.evidencelist) > 0:
		component.value = component.evidencelist[-1]['value']

	r = model.ask(context=["Personal"], view=["lastname"])
	print(r[0].__dict__)
	return component

um.resolverlist["myresolver"] = myresolver

print("now ask for fullname")
reslist = um.ask(context=["Personal"], view='fullname', resolver=dict(resolver="myresolver", evidence_filter="all"))
print("and print result")
printcomplist(reslist, printev = "yes")




#!/usr/bin/env python

import Personis
import Personis_base
from Personis_util import printcomplist

print(">>>> ask for the components in a more complex view of a preference and Alice's first name")
# get component objects for a more complex view
# in this case the first component is "Miles_Davis" in 
# the nominated context, the second component is 
# "firstname" in the "Personal" context
um = Personis.Access(model="Alice", user='alice', password='secret')
reslist = um.ask(context=["Preferences", "Music", "Jazz", "Artists"], 
	view=['Miles_Davis', ['Personal', 'firstname']], resolver={'evidence_filter':"all"})

print("===================================================================")
print(">>>> Now check the evidence list for that view")
printcomplist(reslist, printev = "yes")

res2 = um.ask(context=["Personal"], view=["firstname"], resolver={'evidence_filter':"all"})
print("===================================================================")
printcomplist(res2, printev = "yes")




#!/usr/bin/env python

import Personis_base
from Personis_util import showobj, printcomplist

def printAskContext( info ):
	(cobjlist, contexts, theviews, thesubs) = info
	printcomplist(cobjlist, printev = "yes")
	print("Contexts: %s" % str(contexts))
	print("Views: %s" % str(theviews))
	print("Subscriptions: %s" % str(thesubs))
		
print("===================================================================")
print("Examples that show how app registration works")
print("===================================================================")

um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')

print("List the registered apps (should be none):")
apps = um.listapps()
print(apps)

print("Try and set permissions on a context for an unregistered app:")
try:
	um.setpermission(context=["Personal"], app="MyHealth", permissions={'ask':True, 'tell':False})
except Exception as e:
	print("setpermission failed with exception : %s\n" % (e))

print("Register an app")
appdetails = um.registerapp(app="MyHealth", desc="My Health Manager", password="pass9")
print("Registered ok: ", appdetails)
print("List the registered apps (should be one):")
apps = um.listapps()
print(apps)

print("Set some permissions for the 'MyHealth' app")
um.setpermission(context=["Personal"], app="MyHealth", permissions={'ask':False, 'tell':False, "resolvers":["last1", "goal"]})

print("Show the permissions:")
perms = um.getpermission(context=["Personal"], app="MyHealth")
print("MyHealth:", perms)

print("Try getting permissions for an unregistered app:")
try:
	perms = um.getpermission(context=["Personal"], app="withings")
except Exception as e:
	print("Access failed with exception : %s\n" % (e))
else:
	print("withings:", perms)

print("+++++++++++++++++++++++++++++++++++++++++++++++++++++++")
print("Now testing permissions")
print()
print("Tell full name as owner")
ev = Personis_base.Evidence(evidence_type="explicit", value="Alice")
um.tell(context=["Personal"], componentid='firstname', evidence=ev)
ev = Personis_base.Evidence(evidence_type="explicit", value="Smith")
um.tell(context=["Personal"], componentid='lastname', evidence=ev)

print("Ask for Alice's fullname as owner (should work)")
reslist = um.ask(context=["Personal"], view='fullname')
print(reslist[0].value, reslist[1].value)

um = None
print("Access Alice's model as an unregistered App:")
try:
	um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='withings', password='secret')
except Exception as e:
	print("Access failed with exception : %s\n" % (e))

um = None
print("Access Alice's model as a registered App (should work):")
try:
	um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='MyHealth', password='pass9')
except Exception as e:
	print("Access failed with exception : %s\n" % (e))

print("Ask for Alice's fullname as app 'MyHealth' (should NOT work)")
try:
	reslist = um.ask(context=["Personal"], view='fullname')
	print(reslist[0].value, reslist[1].value)
except Exception as e:
	print("ask failed with exception : %s\n" % (e))

print("Set ask permission for the 'MyHealth' app")
um = None
um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')
um.setpermission(context=["Personal"], app="MyHealth", permissions={'ask':True, 'tell':False})


um = None
um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='MyHealth', password='pass9')
print("Ask for Alice's fullname as app 'MyHealth' (should work now)")
try:
	reslist = um.ask(context=["Personal"], view='fullname')
	print(reslist[0].value, reslist[1].value)
except Exception as e:
	print("ask failed with exception : %s\n" % (e))

print("Now try and tell a new value for first name (should NOT work)")
ev = Personis_base.Evidence(evidence_type="explicit", value="Fred")
try:
	um.tell(context=["Personal"], componentid='firstname', evidence=ev)
except Exception as e:
	print("tell failed with exception : %s\n" % (e))

print("Delete the 'MyHealth' app while NOT accessing as owner")
try:
	um.deleteapp(app="MyHealth")
except Exception as e:
	print("deletapp failed with exception : %s\n" % (e))
else:
	print("FAILED: deleteapp should not be able to delete app when not owner")

um = None
um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')
print("Delete the 'MyHealth' app while accessing as owner")
try:
	um.deleteapp(app="MyHealth")
except Exception as e:
	print("deleteapp failed with exception : %s\n" % (e))
else:
	print("deleteapp succeeded")
print("List the registered apps (should be none):")
apps = um.listapps()
print(apps)



#!/usr/bin/env python

import Personis_base
from Personis_util import showobj, printcomplist

def printAskContext( info ):
	(cobjlist, contexts, theviews, thesubs) = info
	printcomplist(cobjlist, printev = "yes")
	print("Contexts: %s" % str(contexts))
	print("Views: %s" % str(theviews))
	print("Subscriptions: %s" % str(thesubs))
		
print("===================================================================")
print("Simple example showing how permissions work")
print("===================================================================")

um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')

print("Register an app")
appdetails = um.registerapp(app="MyHealth", desc="My Health Manager", password="pass9")
print("Registered ok: ", appdetails)
print("List the registered apps (should be one):")
apps = um.listapps()
print(apps)

print("Set some permissions for the 'MyHealth' app")
um.setpermission(context=["Personal"], app="MyHealth", permissions={'ask':True, 'tell':False})

print("Show the permissions")
perms = um.getpermission(context=["Personal"], app="MyHealth")
print("MyHealth:", perms)
print("Ask for permission for an unregistered app")
try:
	perms = um.getpermission(context=["Personal"], app="withings")
	print("withings:", perms)
except Exception as e:
	print("getpermission failed with exception: %s\n" % (e))

print("Delete the 'MyHealth' app while accessing as owner")
try:
        um.deleteapp(app="MyHealth")
except Exception as e:
        print("deleteapp failed with exception : %s\n" % (e))
else:
        print("deleteapp succeeded")
print("List the registered apps (should be none):")
apps = um.listapps()
print(apps)




#!/usr/bin/env python

import Personis
import Personis_base
from Personis_util import printcomplist, showobj

print(">>>> delete a component")
um = Personis.Access(model="Alice", user='alice', password='secret')

print("before attempting to delete age")
reslist = um.ask(context=["Personal"], view=['age'], resolver={'evidence_filter':"all"})
printcomplist(reslist, printev = "yes")

print("after attempting to delete age")
resd = um.delcomponent(context=["Personal"], componentid = "age")
print(repr(resd))




#!/usr/bin/env python

import Personis_base
from Personis_util import printcomplist

# add some evidence to a component in the music preferences
print("add a preference for Alice for Miles_Davis")
ev = Personis_base.Evidence(evidence_type="explicit", value=4)
um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')
um.tell(context=["Preferences", "Music","Jazz","Artists"], componentid='Miles_Davis', evidence=ev)

print("===================================================================")
print("Now check the evidence list for that preference")
reslist = um.ask(context=["Preferences", "Music", "Jazz", "Artists"], view=['Miles_Davis'], resolver={'evidence_filter':"all"})
printcomplist(reslist, printev = "yes")




#!/usr/bin/env python


# The Personis system is copyright 2000-2012 University of Sydney
#       Bob.Kummerfeld@Sydney.edu.au

# This file is part of Personis.

# Personis is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Personis is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Personis.  If not, see <http://www.gnu.org/licenses/>.

#
# Active User Models: added subscribe method to Access

import base
import subscription
from types import *
import re, time
import cronserver


class Access(base.Access):
	'''Access class with active additions

	:param model: Name of the model to open - must resolve, with modeldir, to a valid directory
	:type model: str
	:param modeldir: Directory of the model
	:type modeldir: str
	:param user: Username of person accessing. Should match model owner, but authorization is handled at server level.
	:type user: str

	'''

	def __init__(self, model=None, modeldir=None, user=None):
		base.Access.__init__(self, model=model, modeldir=modeldir, user=user)
	
	def subscribe(self,
		context=[],
		view=None,
		subscription=None):
		"""

		:param context: is a list giving the path of context identifiers
		:param view: is either:
			an identifier of a view in the context specified
			a list of component identifiers or full path lists
			None indicating that the values of all components in the context be returned
		:param subscription: is a dictionary containing owner, password and subscription statement string
		"""
		subscription['modelname'] = self.modelname
		print "subscription>>> %s %s %s" % (repr(context), repr(view), repr(subscription))
		cronsub = subscription["statement"].strip().startswith("[")
		token = "%s:%f" % (self.user, time.time())
		for elt in ("user", "password", "statement"):
			if elt not in subscription:
					raise ValueError, '"%s" key missing from subscription dict' % (elt)
		if len(subscription.items()) != 4:
				raise ValueError, 'unknown attribute in subscription %s'%(subscription)
		#print "token:", token
		self.curcontext = self._getcontextdir(context)
		contextinfo = self.getcontext(context)
		try:
			comps,comps_shelf_fd = base.shelf_open(self.curcontext+"/.components", "r")
		except:
			print "can't open ",self.curcontext+"/.components"
			comps = None
		try:
			views,views_shelf_fd = base.shelf_open(self.curcontext+"/.views", "r")
		except:
			views = None
		try:
			subs,subs_shelf_fd = base.shelf_open(self.curcontext+"/.subscriptions", "w")
		except:
			subs = None
		cidlist = []
		cobjlist = []
		if type(view) is StringType:
			if views != None:
				if not view in views:
					raise ValueError, '"%s" view not found'%(view)
				cidlist = views[view].component_list
			else:
				raise ValueError, '"%s" view not found'%(view)
		elif type(view) is ListType:
			cidlist = view
		elif view == None: 
			if comps != None:
				cidlist = comps.keys()
		else:
			raise TypeError, 'view "%s" has unknown type'%(repr(view))
		for cid in cidlist:
			if type(cid) == type(u""):
				cid = str(cid)
			if type(cid) is StringType:
				if comps != None:
					if cid in comps:
						# add sub dict to subs for cid ######
						if cid in subs:
							newsub = subs[cid]
						else:
							newsub = {}
						newsub[token] = subscription
						subs[cid] = newsub
						if cronsub:
							cronserver.cronq.put(dict(op="put",context=context, comp=cid, subscription=subscription))
					else:
						raise ValueError, 'component "%s" not in view "%s" (%s)'%(cid,view,cidlist)
				else:
					raise ValueError, 'component "%s" not found'%(cid)
			elif type(cid) is ListType:
				vcontext = self._getcontextdir(cid[:-1])
				try:
					vcomps,vcomps_shelf_fd = base.shelf_open(vcontext+"/.components", "r")
				except:
					raise ValueError, 'context "%s" not in view "%s"'%(cid[-1],repr(view))
				if cid[-1] in vcomps:
					# add sub dict to subs for cid[-1] #########
					if cid[-1] not in subs:
						newsub = {}
					else:
						newsub = subs[cid[-1]]
					newsub[token] = subscription
					subs[cid[-1]] = newsub
					if cronsub:
						cronserver.cronq.put(dict(op="put",context=context, comp=cid, subscription=subscription))
				else:
					raise ValueError, 'component "%s" not in view "%s"'%(cid[-1],repr(view))
				base.shelf_close(vcomps, vcomps_shelf_fd)
					
		if comps != None:
			base.shelf_close(comps, comps_shelf_fd)
		if views != None:
			base.shelf_close(views, views_shelf_fd)
		if subs != None:
			base.shelf_close(subs, subs_shelf_fd)
		return token

	def delete_sub(self, context=None, componentid=None, subname=None):
		self.curcontext = self._getcontextdir(context)
		if type(componentid) == type(u''):
			componentid = str(componentid)
		try:
			subs,subs_shelf_fd = base.shelf_open(self.curcontext+"/.subscriptions", "w")
		except:
			raise ValueError, 'no subs db when deleting sub %s for component "%s" in context "%s" not found'%(subname, componentid,self.curcontext)
		if componentid not in subs:
			raise ValueError, 'sub %s for component "%s" in context "%s" not found'%(subname, componentid,self.curcontext)
		subdict = subs[componentid]
		try:
			del subdict[subname]
		except:
			raise ValueError, 'cannot delete subname "%s" for component "%s" in context "%s" '%(subname,componentid,self.curcontext)
		subs[componentid] = subdict
		if subs != None:
			base.shelf_close(subs, subs_shelf_fd)
		return None

	def list_subs(self, context=None, componentid=None):
		self.curcontext = self._getcontextdir(context)
		try:
			subs,subs_shelf_fd = base.shelf_open(self.curcontext+"/.subscriptions", "r")
		except:
			raise ValueError, 'no subs db when listing subs for component "%s" in context "%s" '%(componentid,self.curcontext)
		if componentid not in subs:
			raise ValueError, 'no subs for component "%s" in context "%s" not found'%(componentid,self.curcontext)
		subdict = subs[componentid]
		if subs != None:
			base.shelf_close(subs, subs_shelf_fd)
		return subdict


	def checksubs(self, context, componentid):
		subs,subs_shelf_fd = base.shelf_open(self.curcontext+"/.subscriptions", "r")
		#print ">>>subs in context '%s': %s" % (self.curcontext, subs.keys())
		#print "checking subs for '%s'"%(componentid)
		if componentid in subs:
			subdict = subs[componentid]
			for subname,sub in subdict.items():
				if sub == {}:
					continue
				subscription.dosub(sub, self)




# Copyright (C) 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for Google App Engine

Utilities for making it easier to use OAuth 2.0 on Google App Engine.
"""

__author__ = 'jcgregorio@google.com (Joe Gregorio)'

import base64
import httplib2
import logging
import pickle
import time

import clientsecrets

from anyjson import simplejson
from client import AccessTokenRefreshError
from client import AssertionCredentials
from client import Credentials
from client import Flow
from client import OAuth2WebServerFlow
from client import Storage
from google.appengine.api import memcache
from google.appengine.api import users
from google.appengine.api import app_identity
from google.appengine.ext import db
from google.appengine.ext import webapp
from google.appengine.ext.webapp.util import login_required
from google.appengine.ext.webapp.util import run_wsgi_app

OAUTH2CLIENT_NAMESPACE = 'oauth2client#ns'


class InvalidClientSecretsError(Exception):
  """The client_secrets.json file is malformed or missing required fields."""
  pass


class AppAssertionCredentials(AssertionCredentials):
  """Credentials object for App Engine Assertion Grants

  This object will allow an App Engine application to identify itself to Google
  and other OAuth 2.0 servers that can verify assertions. It can be used for
  the purpose of accessing data stored under an account assigned to the App
  Engine application itself.

  This credential does not require a flow to instantiate because it represents
  a two legged flow, and therefore has all of the required information to
  generate and refresh its own access tokens.
  """

  def __init__(self, scope, **kwargs):
    """Constructor for AppAssertionCredentials

    Args:
      scope: string or list of strings, scope(s) of the credentials being requested.
    """
    if type(scope) is list:
      scope = ' '.join(scope)
    self.scope = scope

    super(AppAssertionCredentials, self).__init__(
        None,
        None,
        None)

  @classmethod
  def from_json(cls, json):
    data = simplejson.loads(json)
    return AppAssertionCredentials(data['scope'])

  def _refresh(self, http_request):
    """Refreshes the access_token.

    Since the underlying App Engine app_identity implementation does its own
    caching we can skip all the storage hoops and just to a refresh using the
    API.

    Args:
      http_request: callable, a callable that matches the method signature of
        httplib2.Http.request, used to make the refresh request.

    Raises:
      AccessTokenRefreshError: When the refresh fails.
    """
    try:
      (token, _) = app_identity.get_access_token(self.scope)
    except app_identity.Error, e:
      raise AccessTokenRefreshError(str(e))
    self.access_token = token


class FlowProperty(db.Property):
  """App Engine datastore Property for Flow.

  Utility property that allows easy storage and retreival of an
  oauth2client.Flow"""

  # Tell what the user type is.
  data_type = Flow

  # For writing to datastore.
  def get_value_for_datastore(self, model_instance):
    flow = super(FlowProperty,
                 self).get_value_for_datastore(model_instance)
    return db.Blob(pickle.dumps(flow))

  # For reading from datastore.
  def make_value_from_datastore(self, value):
    if value is None:
      return None
    return pickle.loads(value)

  def validate(self, value):
    if value is not None and not isinstance(value, Flow):
      raise db.BadValueError('Property %s must be convertible '
                          'to a FlowThreeLegged instance (%s)' %
                          (self.name, value))
    return super(FlowProperty, self).validate(value)

  def empty(self, value):
    return not value


class CredentialsProperty(db.Property):
  """App Engine datastore Property for Credentials.

  Utility property that allows easy storage and retrieval of
  oath2client.Credentials
  """

  # Tell what the user type is.
  data_type = Credentials

  # For writing to datastore.
  def get_value_for_datastore(self, model_instance):
    logging.info("get: Got type " + str(type(model_instance)))
    cred = super(CredentialsProperty,
                 self).get_value_for_datastore(model_instance)
    if cred is None:
      cred = ''
    else:
      cred = cred.to_json()
    return db.Blob(cred)

  # For reading from datastore.
  def make_value_from_datastore(self, value):
    logging.info("make: Got type " + str(type(value)))
    if value is None:
      return None
    if len(value) == 0:
      return None
    try:
      credentials = Credentials.new_from_json(value)
    except ValueError:
      credentials = None
    return credentials

  def validate(self, value):
    value = super(CredentialsProperty, self).validate(value)
    logging.info("validate: Got type " + str(type(value)))
    if value is not None and not isinstance(value, Credentials):
      raise db.BadValueError('Property %s must be convertible '
                          'to a Credentials instance (%s)' %
                            (self.name, value))
    #if value is not None and not isinstance(value, Credentials):
    #  return None
    return value


class StorageByKeyName(Storage):
  """Store and retrieve a single credential to and from
  the App Engine datastore.

  This Storage helper presumes the Credentials
  have been stored as a CredenialsProperty
  on a datastore model class, and that entities
  are stored by key_name.
  """

  def __init__(self, model, key_name, property_name, cache=None):
    """Constructor for Storage.

    Args:
      model: db.Model, model class
      key_name: string, key name for the entity that has the credentials
      property_name: string, name of the property that is a CredentialsProperty
      cache: memcache, a write-through cache to put in front of the datastore
    """
    self._model = model
    self._key_name = key_name
    self._property_name = property_name
    self._cache = cache

  def locked_get(self):
    """Retrieve Credential from datastore.

    Returns:
      oauth2client.Credentials
    """
    if self._cache:
      json = self._cache.get(self._key_name)
      if json:
        return Credentials.new_from_json(json)

    credential = None
    entity = self._model.get_by_key_name(self._key_name)
    if entity is not None:
      credential = getattr(entity, self._property_name)
      if credential and hasattr(credential, 'set_store'):
        credential.set_store(self)
        if self._cache:
          self._cache.set(self._key_name, credential.to_json())

    return credential

  def locked_put(self, credentials):
    """Write a Credentials to the datastore.

    Args:
      credentials: Credentials, the credentials to store.
    """
    entity = self._model.get_or_insert(self._key_name)
    setattr(entity, self._property_name, credentials)
    entity.put()
    if self._cache:
      self._cache.set(self._key_name, credentials.to_json())

  def locked_delete(self):
    """Delete Credential from datastore."""

    if self._cache:
      self._cache.delete(self._key_name)

    entity = self._model.get_by_key_name(self._key_name)
    if entity is not None:
      entity.delete()


class CredentialsModel(db.Model):
  """Storage for OAuth 2.0 Credentials

  Storage of the model is keyed by the user.user_id().
  """
  credentials = CredentialsProperty()


class OAuth2Decorator(object):
  """Utility for making OAuth 2.0 easier.

  Instantiate and then use with oauth_required or oauth_aware
  as decorators on webapp.RequestHandler methods.

  Example:

    decorator = OAuth2Decorator(
        client_id='837...ent.com',
        client_secret='Qh...wwI',
        scope='https://www.googleapis.com/auth/plus')


    class MainHandler(webapp.RequestHandler):

      @decorator.oauth_required
      def get(self):
        http = decorator.http()
        # http is authorized with the user's Credentials and can be used
        # in API calls

  """

  def __init__(self, client_id, client_secret, scope,
               auth_uri='https://accounts.google.com/o/oauth2/auth',
               token_uri='https://accounts.google.com/o/oauth2/token',
               user_agent=None,
               message=None, **kwargs):

    """Constructor for OAuth2Decorator

    Args:
      client_id: string, client identifier.
      client_secret: string client secret.
      scope: string or list of strings, scope(s) of the credentials being
        requested.
      auth_uri: string, URI for authorization endpoint. For convenience
        defaults to Google's endpoints but any OAuth 2.0 provider can be used.
      token_uri: string, URI for token endpoint. For convenience
        defaults to Google's endpoints but any OAuth 2.0 provider can be used.
      user_agent: string, User agent of your application, default to None.
      message: Message to display if there are problems with the OAuth 2.0
        configuration. The message may contain HTML and will be presented on the
        web interface for any method that uses the decorator.
      **kwargs: dict, Keyword arguments are be passed along as kwargs to the
        OAuth2WebServerFlow constructor.
    """
    self.flow = OAuth2WebServerFlow(client_id, client_secret, scope, user_agent,
        auth_uri, token_uri, **kwargs)
    self.credentials = None
    self._request_handler = None
    self._message = message
    self._in_error = False

  def _display_error_message(self, request_handler):
    request_handler.response.out.write('<html><body>')
    request_handler.response.out.write(self._message)
    request_handler.response.out.write('</body></html>')

  def oauth_required(self, method):
    """Decorator that starts the OAuth 2.0 dance.

    Starts the OAuth dance for the logged in user if they haven't already
    granted access for this application.

    Args:
      method: callable, to be decorated method of a webapp.RequestHandler
        instance.
    """

    def check_oauth(request_handler, *args, **kwargs):
      if self._in_error:
        self._display_error_message(request_handler)
        return

      user = users.get_current_user()
      # Don't use @login_decorator as this could be used in a POST request.
      if not user:
        request_handler.redirect(users.create_login_url(
            request_handler.request.uri))
        return
      # Store the request URI in 'state' so we can use it later
      self.flow.params['state'] = request_handler.request.url
      self._request_handler = request_handler
      self.credentials = StorageByKeyName(
          CredentialsModel, user.user_id(), 'credentials').get()

      if not self.has_credentials():
        return request_handler.redirect(self.authorize_url())
      try:
        method(request_handler, *args, **kwargs)
      except AccessTokenRefreshError:
        return request_handler.redirect(self.authorize_url())

    return check_oauth

  def oauth_aware(self, method):
    """Decorator that sets up for OAuth 2.0 dance, but doesn't do it.

    Does all the setup for the OAuth dance, but doesn't initiate it.
    This decorator is useful if you want to create a page that knows
    whether or not the user has granted access to this application.
    From within a method decorated with @oauth_aware the has_credentials()
    and authorize_url() methods can be called.

    Args:
      method: callable, to be decorated method of a webapp.RequestHandler
        instance.
    """

    def setup_oauth(request_handler, *args, **kwargs):
      if self._in_error:
        self._display_error_message(request_handler)
        return

      user = users.get_current_user()
      # Don't use @login_decorator as this could be used in a POST request.
      if not user:
        request_handler.redirect(users.create_login_url(
            request_handler.request.uri))
        return


      self.flow.params['state'] = request_handler.request.url
      self._request_handler = request_handler
      self.credentials = StorageByKeyName(
          CredentialsModel, user.user_id(), 'credentials').get()
      method(request_handler, *args, **kwargs)
    return setup_oauth

  def has_credentials(self):
    """True if for the logged in user there are valid access Credentials.

    Must only be called from with a webapp.RequestHandler subclassed method
    that had been decorated with either @oauth_required or @oauth_aware.
    """
    return self.credentials is not None and not self.credentials.invalid

  def authorize_url(self):
    """Returns the URL to start the OAuth dance.

    Must only be called from with a webapp.RequestHandler subclassed method
    that had been decorated with either @oauth_required or @oauth_aware.
    """
    callback = self._request_handler.request.relative_url('/oauth2callback')
    url = self.flow.step1_get_authorize_url(callback)
    user = users.get_current_user()
    memcache.set(user.user_id(), pickle.dumps(self.flow),
                 namespace=OAUTH2CLIENT_NAMESPACE)
    return str(url)

  def http(self):
    """Returns an authorized http instance.

    Must only be called from within an @oauth_required decorated method, or
    from within an @oauth_aware decorated method where has_credentials()
    returns True.
    """
    return self.credentials.authorize(httplib2.Http())


class OAuth2DecoratorFromClientSecrets(OAuth2Decorator):
  """An OAuth2Decorator that builds from a clientsecrets file.

  Uses a clientsecrets file as the source for all the information when
  constructing an OAuth2Decorator.

  Example:

    decorator = OAuth2DecoratorFromClientSecrets(
      os.path.join(os.path.dirname(__file__), 'client_secrets.json')
      scope='https://www.googleapis.com/auth/plus')


    class MainHandler(webapp.RequestHandler):

      @decorator.oauth_required
      def get(self):
        http = decorator.http()
        # http is authorized with the user's Credentials and can be used
        # in API calls
  """

  def __init__(self, filename, scope, message=None):
    """Constructor

    Args:
      filename: string, File name of client secrets.
      scope: string or list of strings, scope(s) of the credentials being
        requested.
      message: string, A friendly string to display to the user if the
        clientsecrets file is missing or invalid. The message may contain HTML and
        will be presented on the web interface for any method that uses the
        decorator.
    """
    try:
      client_type, client_info = clientsecrets.loadfile(filename)
      if client_type not in [clientsecrets.TYPE_WEB, clientsecrets.TYPE_INSTALLED]:
        raise InvalidClientSecretsError('OAuth2Decorator doesn\'t support this OAuth 2.0 flow.')
      super(OAuth2DecoratorFromClientSecrets,
            self).__init__(
                client_info['client_id'],
                client_info['client_secret'],
                scope,
                client_info['auth_uri'],
                client_info['token_uri'],
                message)
    except clientsecrets.InvalidClientSecretsError:
      self._in_error = True
    if message is not None:
      self._message = message
    else:
      self._message = "Please configure your application for OAuth 2.0"


def oauth2decorator_from_clientsecrets(filename, scope, message=None):
  """Creates an OAuth2Decorator populated from a clientsecrets file.

  Args:
    filename: string, File name of client secrets.
    scope: string or list of strings, scope(s) of the credentials being
      requested.
    message: string, A friendly string to display to the user if the
      clientsecrets file is missing or invalid. The message may contain HTML and
      will be presented on the web interface for any method that uses the
      decorator.

  Returns: An OAuth2Decorator

  """
  return OAuth2DecoratorFromClientSecrets(filename, scope, message)


class OAuth2Handler(webapp.RequestHandler):
  """Handler for the redirect_uri of the OAuth 2.0 dance."""

  @login_required
  def get(self):
    error = self.request.get('error')
    if error:
      errormsg = self.request.get('error_description', error)
      self.response.out.write(
          'The authorization request failed: %s' % errormsg)
    else:
      user = users.get_current_user()
      flow = pickle.loads(memcache.get(user.user_id(),
                                       namespace=OAUTH2CLIENT_NAMESPACE))
      # This code should be ammended with application specific error
      # handling. The following cases should be considered:
      # 1. What if the flow doesn't exist in memcache? Or is corrupt?
      # 2. What if the step2_exchange fails?
      if flow:
        credentials = flow.step2_exchange(self.request.params)
        StorageByKeyName(
            CredentialsModel, user.user_id(), 'credentials').put(credentials)
        self.redirect(str(self.request.get('state')))
      else:
        # TODO Add error handling here.
        pass


application = webapp.WSGIApplication([('/oauth2callback', OAuth2Handler)])


def main():
  run_wsgi_app(application)



#!/usr/bin/env python

import sys
import Personis
import Personis_base
import Personis_a
from Personis_util import printcomplist, printjson

print("skipping import test. Edit script to test")
sys.exit(0)

# import a model sub tree from JSON

um = Personis_a.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')
modelfile = "saved-model"
print("importing:", modelfile)
modeljson = open(modelfile).read()
um.import_model(partial_model=modeljson)




#!/usr/bin/env python

import Personis
import Personis_base
from Personis_util import showobj, printcomplist

print(">>>> create a new view in a given context")
um = Personis.Access(model='Alice', user='alice', password='secret')
vobj = Personis_base.View(Identifier="email_details", component_list=["firstname", "lastname", "email"])

print(">>>> view object with Alice's email details, names and email address")
showobj(vobj, 1)

um.mkview(context=["Personal"], viewobj=vobj)

reslist= um.ask(context=["Personal"], view = 'email_details', resolver={'evidence_filter':"all"})
printcomplist(reslist)



#!/usr/bin/env python

import Personis_base
from Personis_util import printcomplist, showobj

print("Testing goals")
print("\tcreate a new component")
um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')
cobj = Personis_base.Component(Identifier="fitness", component_type="goal", Description="My overall fitness", goals=[['Personal', 'Health', 'weight']], value_type="number")
showobj(cobj, 1)

res = um.mkcomponent(context=["Personal"], componentobj=cobj)
print(repr(res))

print("\t show this component in the Personal context")
ev = Personis_base.Evidence(evidence_type="explicit", value=17)
um.tell(context=["Personal"], componentid='fitness', evidence=ev)
reslist = um.ask(context=["Personal"], view=['fitness'], resolver={'evidence_filter':"all"})
printcomplist(reslist, printev = "yes")

goals = reslist[0].goals
print("Old goals are:", goals)
print("\tadding a goal")
goals.append(["Personal", "Health", "steps"])
um.set_goals(context=["Personal"], componentid='fitness', goals=goals)

reslist = um.ask(context=["Personal"], view=['fitness'], resolver={'evidence_filter':"all"})
printcomplist(reslist, printev = "yes")





# Copyright (C) 2010 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command-line tools for authenticating via OAuth 2.0

Do the OAuth 2.0 Web Server dance for a command line application. Stores the
generated credentials in a common file that is used by other example apps in
the same directory.
"""

__author__ = 'jcgregorio@google.com (Joe Gregorio)'
__all__ = ['run']


import BaseHTTPServer
import gflags
import socket
import sys
import webbrowser

from client import FlowExchangeError
from client import OOB_CALLBACK_URN

try:
  from urlparse import parse_qsl
except ImportError:
  from cgi import parse_qsl


FLAGS = gflags.FLAGS

gflags.DEFINE_boolean('auth_local_webserver', True,
                      ('Run a local web server to handle redirects during '
                       'OAuth authorization.'))

gflags.DEFINE_string('auth_host_name', 'localhost',
                     ('Host name to use when running a local web server to '
                      'handle redirects during OAuth authorization.'))

gflags.DEFINE_multi_int('auth_host_port', [8080, 8090],
                        ('Port to use when running a local web server to '
                         'handle redirects during OAuth authorization.'))


class ClientRedirectServer(BaseHTTPServer.HTTPServer):
  """A server to handle OAuth 2.0 redirects back to localhost.

  Waits for a single request and parses the query parameters
  into query_params and then stops serving.
  """
  query_params = {}


class ClientRedirectHandler(BaseHTTPServer.BaseHTTPRequestHandler):
  """A handler for OAuth 2.0 redirects back to localhost.

  Waits for a single request and parses the query parameters
  into the servers query_params and then stops serving.
  """

  def do_GET(s):
    """Handle a GET request.

    Parses the query parameters and prints a message
    if the flow has completed. Note that we can't detect
    if an error occurred.
    """
    s.send_response(200)
    s.send_header("Content-type", "text/html")
    s.end_headers()
    query = s.path.split('?', 1)[-1]
    query = dict(parse_qsl(query))
    s.server.query_params = query
    s.wfile.write("<html><head><title>Authentication Status</title></head>")
    s.wfile.write("<body><p>The authentication flow has completed.</p>")
    s.wfile.write("</body></html>")

  def log_message(self, format, *args):
    """Do not log messages to stdout while running as command line program."""
    pass


def run(flow, storage, http=None):
  """Core code for a command-line application.

  Args:
    flow: Flow, an OAuth 2.0 Flow to step through.
    storage: Storage, a Storage to store the credential in.
    http: An instance of httplib2.Http.request
         or something that acts like it.

  Returns:
    Credentials, the obtained credential.
  """
  if FLAGS.auth_local_webserver:
    success = False
    port_number = 0
    for port in FLAGS.auth_host_port:
      port_number = port
      try:
        httpd = ClientRedirectServer((FLAGS.auth_host_name, port),
                                     ClientRedirectHandler)
      except socket.error, e:
        pass
      else:
        success = True
        break
    FLAGS.auth_local_webserver = success
    if not success:
      print 'Failed to start a local webserver listening on either port 8080'
      print 'or port 9090. Please check your firewall settings and locally'
      print 'running programs that may be blocking or using those ports.'
      print
      print 'Falling back to --noauth_local_webserver and continuing with',
      print 'authorization.'
      print

  if FLAGS.auth_local_webserver:
    oauth_callback = 'http://%s:%s/' % (FLAGS.auth_host_name, port_number)
  else:
    oauth_callback = OOB_CALLBACK_URN
  authorize_url = flow.step1_get_authorize_url(oauth_callback)

  if FLAGS.auth_local_webserver:
    webbrowser.open(authorize_url, new=1, autoraise=True)
    print 'Your browser has been opened to visit:'
    print
    print '    ' + authorize_url
    print
    print 'If your browser is on a different machine then exit and re-run this'
    print 'application with the command-line parameter '
    print
    print '  --noauth_local_webserver'
    print
  else:
    print 'Go to the following link in your browser:'
    print
    print '    ' + authorize_url
    print

  code = None
  if FLAGS.auth_local_webserver:
    httpd.handle_request()
    if 'error' in httpd.query_params:
      sys.exit('Authentication request was rejected.')
    if 'code' in httpd.query_params:
      code = httpd.query_params['code']
    else:
      print 'Failed to find "code" in the query parameters of the redirect.'
      sys.exit('Try running with --noauth_local_webserver.')
  else:
    code = raw_input('Enter verification code: ').strip()

  try:
    credential = flow.step2_exchange(code, http)
  except FlowExchangeError, e:
    sys.exit('Authentication has failed: %s' % e)

  storage.put(credential)
  credential.set_store(storage)
  print 'Authentication successful.'

  return credential



#!/usr/bin/env python

import unittest
import Personis_base
import os
import shutil
import sys

class TestPersonisBaseAdd(unittest.TestCase):

    def setUp(self):
        # create model
        shutil.copytree('models', 'testmodels')

    def tearDown(self):
        shutil.rmtree('testmodels')

    def test_add(self):
        um = Personis_base.Access(model="Alice", modeldir='testmodels', user='alice', password='secret')
        # create a piece of evidence with Alice as value
        ev = Personis_base.Evidence(evidence_type="explicit", value="Alice")
        # tell this as user alice's first name
        um.tell(context=["Personal"], componentid='firstname', evidence=ev)
        ev = Personis_base.Evidence(evidence_type="explicit", value="Smith")
        um.tell(context=["Personal"], componentid='lastname', evidence=ev)
        ev = Personis_base.Evidence(evidence_type="explicit", value="female")
        um.tell(context=["Personal"], componentid='gender', evidence=ev)

        reslist = um.ask(context=["Personal"], view='fullname', resolver={'evidence_filter':"last1"})
        self.assertEqual(reslist[0].value, 'Alice')
        reslist = um.ask(context=["Personal"], view=['gender'], resolver={'evidence_filter':"last1"})
        self.assertEqual(reslist[0].value, 'female')

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import Personis_base 
from Personis_util import showobj, printcomplist


um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')


for i in range(100000):
	res = um.ask(context=["Personal"], view=['gender'])



#!/usr/local/bin/python

import Personis
import Personis_a

print(">>>> delete a subscription (temp test)")


um = Personis.Access(model='Alice', user='alice', password='secret')

reslist = um.ask(context=["Personal"], view='fullname', showcontexts=True)
print(reslist)
cobjlist, contexts, theviews, thesubs = reslist
subname = list(thesubs.items())[0][1].items()[0][0]
print(">>>> first sub is: ", subname)

result = um.delete_sub(context=["Personal"], componentid='lastname', subname=subname)

print(">>>> after deletion")
print(result)





#!/usr/bin/env python

from personis import client

import httplib2
import os
from collections import defaultdict


# get past the uni's stupid proxy server
p = httplib2.ProxyInfo(proxy_type=httplib2.socks.PROXY_TYPE_HTTP_NO_TUNNEL, proxy_host='www-cache.it.usyd.edu.au', proxy_port=8000)

# Use the util package to get a link to UM. This uses the client_secrets.json file for the um location
client_secrets = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'client_secrets.json')
um = client.util.LoginFromClientSecrets(filename=client_secrets, http=httplib2.Http(proxy_info=p, disable_ssl_certificate_validation=True), credentials='asker_cred.dat')

# Ask UM for the logged items
reslist = um.ask(context=["Apps", 'Logging'],view=['logged_items'])

# Go through the returned evidence and count the number of times each item was logged
d = defaultdict(int)
for res in reslist:
	evlist = res.evidencelist
	for ev in evlist:
		d[ev.value] += 1

# get the length of the longest string in the keys of d. Don't try this at home.
leng = len(reduce(lambda x, y: x if len(x) > len(y) else y, d.keys()))
# use it to create a format string
f = '{:>'+repr(leng + 1)+'} : {}'
# print the dict using the format
for k, v in d.items():
	print f.format(k, '*'*v)


#!/usr/bin/env python

# The Personis system is copyright 2000-2012 University of Sydney
#       Bob.Kummerfeld@Sydney.edu.au

# This file is part of Personis.

# Personis is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Personis is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Personis.  If not, see <http://www.gnu.org/licenses/>.

"""

mkmodel - a program to create a set of models
usage: mkmodel modeldefinitionfile modeldirectory modelname modelname ...

mkmodel takes a definition of a model (stored in modeldefinitionfile)
and creates a model in modeldirectory for each model 

see external documentation for the definition of the modeldef language

"""

import sys
from Personis_mkmodel import *


def doit():
	if len(sys.argv) < 4 or '--modelserver'== sys.argv[-1]:
		print "usage: mkmodel modelfile modeldir model-name ...."
		print "or: mkmodel --modelserver modelservername:port modelfile model-name ...."
		sys.exit(0)

	modelserver = None
	if len(sys.argv) > 4 and '--modelserver' in sys.argv[:-1]:
		ind = sys.argv.index('--modelserver')
		modelserver = sys.argv[ind+1]
		del sys.argv[ind+1]
		del sys.argv[ind]
		for m in sys.argv[2:]:
			mname,user,password = m.split(":")
			mkmodel_remote(model=mname, mfile=sys.argv[1], modelserver=modelserver, user=user, password=password)
	else:
		for m in sys.argv[3:]:
			mname,user,password = m.split(":")
			print mname, sys.argv[1], sys.argv[2], user, password
			mkmodel(model=mname, mfile=sys.argv[1], modeldir=sys.argv[2], user=user, password=password)


if __name__ == '__main__':
	doit()
	



# The Personis system is copyright 2000-2012 University of Sydney
#       Bob.Kummerfeld@Sydney.edu.au

# This file is part of Personis.

# Personis is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Personis is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Personis.  If not, see <http://www.gnu.org/licenses/>.

#	David Carmichael 
# 	July 2006
#

''' Exceptions used in the Personis System '''

class ModelNotFoundError(IOError):
	"Used to indicate that no model can be found for given modelname"

class ModelServerError(IOError):
	"Used to indicate that the required model server is returning an error"





#!/usr/bin/env python

import time
import sys
from personis import client
import json
from optparse import OptionParser
import yaml
import httplib2
import os
import gflags
import logging


import cmd

def printjson(jsonobj):
    json.dump(sysjson.loads(jsonobj), sys.stdout, sort_keys=True, indent=4)
    print

def loggedin(modelserver, credentials, cmd):
    if (credentials == None):
        print cmd, ": please login"
        return False
    return True

def nomodel(modelname,cmd):
    if modelname == "":
        print cmd, ": please select a model using the 'model' command"
        return False
    return True

def lscontext(um, cont):
    try:
        info = um.ask(context=cont, showcontexts=True)
        (cobjlist, contexts, theviews, thesubs) = info
        #printcomplist(cobjlist, printev = "yes")
        print "Components:"
        for cobj in cobjlist:
            print "\t%s: %s"%(cobj.Identifier, cobj.Description)
        print "Contexts: %s" % str(contexts)
        print "Views: %s" % str(theviews)
        print "Subscriptions: %s" % str(thesubs)
    except ValueError, e:
        print "ask failed: %s" % (e)

class browse(cmd.Cmd):

    def __init__(self):
        self.accesstype = "base"
        self.attrs = {"voluble":False, "evnum":1}
        self.um = None
        self.username = ''
        self.context = [""]
        self.intro = "Personis Model Browser"
        self.prompt = "%s > " %(self.context)
        cmd.Cmd.__init__(self)

    def do_set(self, line):
        """
        usage: set atrr-name value
        gives an attribute a value, eg
                set evnum 5
        will set the number of evidence items to print to 5
        """
        line = line.split()
        if len(line) != 2:
            print "usage: set atrr-name value"
            return
        if line[0] not in self.attrs:
            print "illegal attribute"
            return
        val = line[1]
        if line[1] == "True":
            val = True
        if line[1] == "False":
            val = False
        self.attrs[line[0]] = val

    def do_attributes(self, line):
        """attributes lists current attributes
                use the set command to set an attribute"""
        print self.attrs

    def do_model(self, line):
        """model modelname [modeldirectory]"""
        return

        line = line.split()
        if len(line) == 0:
            print "usage: model modelname"
            return
        self.umname = line[0]
        try:
            self.um = client.Access(modelserver=self.modelserver, credentials=self.credentials)
        except:
            print "Failed to access model '%s', user '%s', password '%s'" % (self.umname, self.username, self.userpassword)
            return

        print "model '%s' open, access type is '%s'" % (self.umname, self.accesstype)

    def do_quit(self, line):
        """quit the model browser"""
        print "Goodbye!"
        sys.exit(0)

    def do_ls(self, line):
        """ls contextname|componentname"""
        line = line.split()
        cont = self.context
        if len(line) == 1:
            if line[0][-1] == '/':
                lscontext(self.um, self.context+[line[0][:-1]])
            else:
                try:
                    info = self.um.ask(context=self.context, view=[line[0]], resolver=dict(evidence_filter="all"))
                    client.util.PrintComplist(info, printev="yes", count=int(self.attrs["evnum"]))
                except ValueError, e:
                    print "ask failed: %s" % (e)
        else:
            lscontext(self.um, self.context)

    def do_tell(self, line):
        """tell component_name
                will prompt for component value etc"""
        line = line.split()
        if len(line) != 1:
            print "usage: tell component"
            return
        compname = line[0]
        val = raw_input("Value? ")
        print "Evidence type:"
        for et in client.Evidence.EvidenceTypes:
            print client.Evidence.EvidenceTypes.index(et), et
        etindex = raw_input("Index? [0]")
        if etindex == '':
            etindex = 0
        else:
            etindex = int(etindex)
        if (etindex < 0) or (etindex > len(client.Evidence.EvidenceTypes)-1):
            print "Index out of range"
            return
        etype = client.Evidence.EvidenceTypes[etindex]
        source = self.username
        flags = []
        while True:
            flag = raw_input("Evidence flag? (CR for none)")
            if flag == '':
                break
            flags.append(flag)

        print "Tell value=%s, type=%s, flags=%s, source=%s, context=%s, component=%s " % (val, etype, flags, source, self.context, compname)
        ok = raw_input("Ok?[N] ")
        if ok != 'Y':
            return
        ev = client.Evidence(source=source, evidence_type=etype, flags=flags, value=val)
        try:
            self.um.tell(context=self.context, componentid=compname, evidence=ev)
        except ValueError, e:
            print "tell failed: %s" % (e)



    def do_cd(self, line):
        """cd context_name
                changes to a subcontext or parent context using .. """

        line = line.split()
        savecontext = list(self.context)
        if len(line) == 1:
            if line[0] == '..':
                if len(self.context) > 1:
                    self.context = self.context[:-1]
                else:
                    self.context = ['']
            else:
                self.context.append(line[0])
        else:
            self.context = ['']
        try:
            info = self.um.ask(context=self.context)
        except:
            print "context not found:", self.context
            self.context = savecontext
        self.prompt = "%s > " %(self.context)

    def do_login(self, line):
        """login username password"""
        return

        line = line.split()
        if len(line) == 2:
            self.username = line[0]
            self.userpassword = line[1]
        else:
            print "usage: login user password"
        print "username:", self.username
        print "password:", self.userpassword

    def do_base(self, line):
        """sets the access type to a model in the local machine file system"""
        self.accesstype = "base"

    def do_server(self, line):
        """sets the access type to a model in a remote server"""
        self.accesstype = "server"

    def do_export(self, line):
        """export filename
                exports current context to the given file in JSON form"""
        line = line.split()
        if len(line) != 1:
            print "usage: export tofilename"
            return
        try:
            expfile = open(line[0], "w")
        except:
            print "export: cannot open ", line[0]
            return

        modeljson = self.um.export_model(context=self.context, resolver = {'evidence_filter': "all"})
        mm = json.loads(modeljson)
        expfile.write(json.dumps(mm,sort_keys=True,indent=4))
        if self.attrs["voluble"]:
            print modeljson

    def do_import(self, line):
        """import fromfilename
                imports a model from the given file in JSON format"""
        line = line.split()
        if len(line) != 1:
            print "usage: import fromfilename"
            return
        try:
            importfile = open(line[0], "r")
        except:
            print "import: cannot open ", line[0]
            return
        self.um.import_model(context=self.context, partial_model=importfile.read())
        importfile.close()

    #def do_importmdef(self, line):
    #    """importmdef fromfilename
    #            imports a model in modeldef format"""
    #    line = line.split()
    #    if len(line) != 1:
    #        print "usage: importmdef fromfilename"
    #        return
    #    Personis_mkmodel.mkmodel_um(um,Personis_mkmodel.get_modeldef(line[0]))
        #try:
        #       Personis_mkmodel.mkmodel_um(um,Personis_mkmodel.get_modeldef(line[0]))
        #except:
        #       print "import modeldef failed"

    def do_setgoals(self, line):
        """setgoals componentname goal-component [goal-component ...] """
        line = line.split()
        if len(line) < 2:
            print "usage: setgoals componentname goal-component [goal-component ...] "
            return
        comp = line[0]
        goals = [g.split('/') for g in line[1:]]
        self.um.set_goals(context=self.context, componentid=comp, goals=goals)

    def do_app(self, line):
        """app application_name
                register a new app"""
        line = line.split()
        if len(line) != 1:
            print "No new app specified"
            print "usage: app application_name"
            print "Registered applications:"
            print self.um.listapps()
            return
        appname = line[0]
        appdesc = raw_input("Application description: ")
        app_pass = raw_input("Application password: ")
        print self.um.registerapp(app=appname, desc=appdesc, password=app_pass)

    def do_setperm(self, line):
        """setperm application_name
                set permissions for an application"""
        line = line.split()
        if len(line) != 1:
            print "usage: setperm appname"
            return
        appname = line[0]
        apps = self.um.listapps()
        if not (appname in apps):
            print "app '%s' not registered"
            return
        contxt = self.context
        tellperm = raw_input("Tell ok? [N] ")
        if tellperm == '':
            tellperm = False
        elif tellperm == 'Y':
            tellperm = True
        elif tellperm == 'N':
            tellperm = False
        else:
            print "answer Y or N or enter for default"
        askperm = raw_input("Ask ok? [N] ")
        if askperm == '':
            askperm = False
        elif askperm == 'Y':
            askperm = True
        elif askperm == 'N':
            askperm = False
        else:
            print "answer Y or N or enter for default"
        print "Setting permission for app '%s': Ask=%s, Tell=%s  in context %s" % (appname, askperm, tellperm, contxt)
        ok = raw_input("Ok? [N] ")
        if ok != 'Y':
            print "Permissions NOT set"
            return
        self.um.setpermission(context=contxt, app=appname, permissions={'ask':askperm, 'tell':tellperm} )
        print "Permissions set"

    def do_showperm(self, line):
        """showperm application_name
                displays permission information for an application"""
        line = line.split()
        if len(line) != 1:
            print "usage: showperm appname"
            return
        appname = line[0]
        apps = self.um.listapps()
        if not (appname in apps):
            print "app '%s' not registered"
            return
        contxt = raw_input("Context path: ")
        contxt = contxt.split('/')
        print "Permissions for app '%s' in context %s:" % (appname, contxt)
        print self.um.getpermission(context=contxt, app=appname)

    def do_subscribe(self, line):
        """subscribe component_name
                add a subscription rule to a component"""
        line = line.split()
        if len(line) != 1:
            print "usage: subscribe component-name"
            return
        contxt = self.context
        comp = line[0]
        sub = raw_input("Subscription statement: ")
        print "Subscribing for component '%s' in context %s, with statement:\n%s" % (comp, contxt, sub)
        sub = {'user':self.username, 'password':'', 'statement':sub}
        ok = raw_input("Ok? [N] ")
        if ok != 'Y':
            return
        self.um.subscribe(context=contxt, view=[comp], subscription=sub)

    def do_delsub(self, line):
        """delsub component_name
                delete a subscription rule from a component"""

        line = line.split()
        if len(line) != 1:
            print "usage: delsub compname"
            return
        contxt = self.context
        comp = line[0]
        print "Deleting subscriptions in context %s, component '%s'" % (contxt, comp)
        reslist = self.um.ask(context=contxt, view=[comp], showcontexts=True)
        cobjlist, contexts, theviews, thesubs = reslist
        thesubs = thesubs[comp]
        if thesubs == {}:
            print "Component '%s' has no subscriptions" % (comp)
            return
        for sid, sub in thesubs.items():
            d = raw_input("delete sub '%s'?[N] "%sub)
            if d == 'Y':
                result = self.um.delete_sub(context=contxt, componentid=comp, subname=sid)
                print result

    def do_mkcomponent(self, line):
        """mkcomponent component_name
                make a new component"""

        line = line.split()
        if len(line) != 1:
            print "usage: mkcomponent compname"
            return
        contxt = self.context
        comp = line[0]
        compdesc = raw_input("Component description? ")
        print "Component type:"
        for ct in client.Component.ComponentTypes:
            print client.Component.ComponentTypes.index(ct), ct
        ctindex = int(raw_input("Index? "))
        if (ctindex < 0) or (ctindex > len(client.Component.ComponentTypes)-1):
            print "Index out of range"
            return
        comptype = client.Component.ComponentTypes[ctindex]

        print "Value type:"
        for ct in client.Component.ValueTypes:
            print client.Component.ValueTypes.index(ct), ct
        ctindex = int(raw_input("Index? "))
        if (ctindex < 0) or (ctindex > len(client.Component.ValueTypes)-1):
            print "Index out of range"
            return
        valtype = client.Component.ValueTypes[ctindex]

        print "Creating new component '%s', type '%s', description '%s', value type '%s'" % (comp, comptype, compdesc, valtype)
        ok = raw_input("Ok?[N] ")
        if ok != 'Y':
            return
        cobj = client.Component(Identifier=comp, component_type=comptype, Description=compdesc, value_type=valtype)
        res = self.um.mkcomponent(context=contxt,  componentobj=cobj)

    def do_delcomponent(self, line):
        """delcomponent component_name
                delete a component"""
        line = line.split()
        if len(line) != 1:
            print "usage: delcomponent compname"
            return
        comp = line[0]
        print "Deleting component '%s' in context %s" % (comp, self.context)
        ok = raw_input("Ok?[N] ")
        if ok != 'Y':
            return
        res = self.um.delcomponent(context=self.context, componentid=comp)
        if res:
            print res


    def do_mkcontext(self, line):
        """mkcontext context_name
                make a new context"""

        line = line.split()
        if len(line) != 1:
            print "usage: mkcontext contextname"
            return
        newcontext = line[0]
        contextdesc = raw_input("Context description? ")
        print "Create new context '%s' in context '%s' with description '%s'" % (newcontext, self.context, contextdesc)
        ok = raw_input("Ok?[N] ")
        if ok != 'Y':
            return
        cobj = client.Context(Identifier=newcontext, Description=contextdesc)
        if not self.um.mkcontext(self.context, cobj):
            print "Failed to make context. May be already present"

    def do_listapps(self, line):
        """listapps
                list registered applications"""

        print self.um.listapps()

    def do_delapp(self, line):
        """delapp appname
                delete an application
        """

        line = line.split()
        if len(line) > 1:
            print "usage: delapp [name]"
            return
        if len(line) == 1:
            self.um.deleteapp(line[0])
            return
        applist = self.um.listapps()
        for appname in applist:
            ok = raw_input("Delete app '%s'? [N] " % (appname))
            if ok == 'Y':
                self.um.deleteapp(appname)

    def do_mkmodel(self, line):
        """mkmodel model_name [model_directory]
                makes a new empty model
                uses username, password specified in login cmd"""
        return
        line = line.split()
        if self.accesstype == "base":
            if len(line) != 2:
                print "usage: mkmodel model_name model_directory"
                return
            umname = line[0]
            modeldir = line[1]
            print "making model '%s' in directory '%s' with username '%s' and password '%s'" % (self.umname, modeldir, self.username, self.userpassword)
            try:
                client.MkModel(model=self.umname, modeldir=modeldir, user=self.username, password=self.userpassword)
            except Exception, e:
                print "mkmodel failed: ", e
                return
            print "model made"
            print "to access the model, use the command 'model %s %s'" % (self.umname, modeldir)
        else:
            print "can only make models locally at present"

    def do_do(self, line):
        """do filename
                execute browser commands from a file, then switch back to interactive mode"""
        line = line.split()
        if len(line) != 1:
            print "usage: do cmdfile"
        else:
            with open(line[0]) as f:
                input = [x for x in f.read().splitlines() if x[0] != "#"]
            for line in input:
                print ">>%s" % (line)
            self.cmdqueue.extend(input)

# The gflags module makes defining command-line options easy for
# applications. Run this program with the '--help' argument to see
# all the flags that it understands.
gflags.DEFINE_enum('logging_level', 'ERROR',
    ['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
    'Set the level of logging detail.')

# Let the gflags module process the command-line arguments
try:
    argv = gflags.FLAGS(sys.argv)
except gflags.FlagsError, e:
    print '%s\\nUsage: %s ARGS\\n%s' % (e, sys.argv[0], FLAGS)
    sys.exit(1)

# Set the logging according to the command-line flag
logging.getLogger().setLevel(getattr(logging, gflags.FLAGS.logging_level))

b = browse()
# get past the uni's stupid proxy server
p = httplib2.ProxyInfo(proxy_type=httplib2.socks.PROXY_TYPE_HTTP_NO_TUNNEL, proxy_host='www-cache.it.usyd.edu.au', proxy_port=8000)

# Use the util package to get a link to UM. This uses the client_secrets.json file for the um location
client_secrets = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'client_secrets.json')
b.um = client.util.LoginFromClientSecrets(filename=client_secrets, http=httplib2.Http(proxy_info=p, disable_ssl_certificate_validation=True), credentials='browser_cred.dat')
reslist = b.um.ask(context=["Personal"],view=['firstname'])
b.username = reslist[0].value
print 'Welcome', b.username

b.cmdloop()




#!/bin/env python


# The Personis system is copyright 2000-2012 University of Sydney
#       Bob.Kummerfeld@Sydney.edu.au

# This file is part of Personis.

# Personis is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Personis is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Personis.  If not, see <http://www.gnu.org/licenses/>.

"""
	Functions for handling subscriptions

	Uses the pyparsing module to parse and execute the subscription statement.
	
	Run as a standalone program for testing.


	Subscription statement grammar:

		resolvername := ID
		evidencetype := ID
		componentval := '<' [ resolvername '!' ] component '>'
		componentname := ID | componentval
		modelname := '.' | componentname
		component := modelname [ '/' componentname ]*
		cronspec := '[' QUOTEDSTRING ']'
		item := component | QUOTEDSTRING
		patternmatch := item '~' item
		tell := 'TELL' item ',' evidencetype ':' item
		tellchanged := 'TELLCHANGED' item ',' evidencetype ':' item
		notify := 'NOTIFY' item [item]*
		action := tell | tellchanged | notify
		statement := [ cronspec ] patternmatch ':' action

	Note:
		a cronspec string is similar to a crontab entry: "minute hour day_of_month month day_of_week"

	Examples:

		 <froomBT/seen> ~ '.*' : TELL <froomBT/seen>/location, <froomBT/location>"
		 <froomBT/seen> ~ '.*' : NOTIFY 'http://www/qqq.cgi'"
		 <froomBT/seen> ~ '.*' : TELL bob/personal/location, explicit:<froomBT/location>
		 <bob/personal/location> ~ '.*' : NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi'
		 ["*/15 * * * *"] <bob/personal/location> ~ '.*' : NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi'
		 <default!bob/personal/location> ~ '.*' : NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi'
		 <default!bob/personal/location> ~ '.*' : NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi'
		 <bobPhone/seenby> ~ '.*' : TELL bobPhone/location, explicit:<<bobPhone/seenby>/location>
		 <default!./personal/location> ~ '.*' : NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi?' 'location=' <./personal/location>  '&name=' <./personal/firstname>

"""
#import Personis_server
from . import base
import re
from .mypyparsing import Word, alphas, Literal, ZeroOrMore, quotedString, Forward, removeQuotes, ParseException, Optional, OneOrMore


def askval(str, loc, toks):
	if toks[2] == '!': 
		resolver = toks[1]
		del toks[1:3]
	else:
		resolver = "default"
	model = toks[1]
	if model == '.': model = defaultmodel
	context = [x for x in toks[2:-3] if x != '/']
	componentid = toks[-2]
	print "AskVal -> resolver: %s, model: %s, context: %s, componentid:%s" % (resolver, model, context, componentid)
	if model != defaultmodel:
		global Personis
		import Personis as pmdns
		try:
			um = pmdns.Access(model=model, user=user, password=password)
			reslist = um.ask(context=context, view=[componentid], resolver=resolver)
		except:
			print "ask failed"
			return "-no model-"
	else:
		reslist = currentum.ask(context=context, view=[componentid], resolver=resolver)
	print "result: ", reslist[0].value
	if reslist[0].value == None:
		return ""
	else:
		return reslist[0].value

def domatch(str, loc, toks):
	print "TOKS:", toks
	if re.compile(toks[2]).match(toks[0]):
		return "-match-"
	else:
		raise ParseException(str, loc, "match failed",0)
		
def dotellchanged(str, loc, toks):
	return dotell_body(str, loc, toks, changed_only=True)

def dotell(str, loc, toks):
	return dotell_body(str, loc, toks)

def dotell_body(str, loc, toks,changed_only=False):
	model = toks[1]
	if model == '.': model = defaultmodel
	context = [x for x in toks[2:-6] if x != '/']
	componentid = toks[-5]
	evidence_type = toks[-3]
	newval = toks[-1]
	
	print "Tell %s/%s/%s, %s:%s" % (model, context, componentid, evidence_type, newval)
	try:
		if model!=defaultmodel:
			global Personis
			import Personis as pmdns
			um = pmdns.Access(model=model, user=user, password=password)
			if changed_only:
				comp = um.ask(context=context, view=[componentid], resolver='default')[0]
				if comp.value == newval:
					return True # don't actually need to do the tell
			um.tell(context=context, componentid=componentid,
				evidence=base.Evidence(evidence_type=evidence_type, value=newval))
		else:
			if changed_only:
				comp = currentum.ask(context=context, view=[componentid], resolver='default')[0]
				if comp.value == newval:
					return True # don't actually need to do the tell
			currentum.tell(context=context, componentid=componentid,
				evidence=base.Evidence(evidence_type=evidence_type, value=newval))

	except:
		print 'tell failed'
		raise ParseException(str, loc, "tell failed",0)
	return True

def donotify(str, loc, toks):
	#print "donotify::", toks
	url = "".join(toks[1:])
	import urllib.request, urllib.parse, urllib.error
	print "Notify", url
	f = urllib.request.urlopen(url)
	print f.readlines()
	f.close()
	return

component = Forward()
componentval = Literal("<")+Optional(Word(alphas) + Literal('!'))+component+Literal(">")
componentval.setParseAction(askval)
componentname = Word(alphas) | componentval
modelname = Literal('.') | componentname
component << modelname + ZeroOrMore(Literal('/') + componentname)
item = component | quotedString.setParseAction(removeQuotes)
cronspec = Literal('[') + quotedString.setParseAction(removeQuotes) + Literal(']')
patternmatch = item + Literal('~') + item 
patternmatch.setParseAction(domatch)
tell = Literal('TELL') + item + Literal(',') + item + Literal(':') + item
tell.setParseAction(dotell)
tellchanged = Literal('TELLCHANGED') + item + Literal(',') + item + Literal(':') + item
tellchanged.setParseAction(dotellchanged)
notify = Literal('NOTIFY') + OneOrMore(item)
notify.setParseAction(donotify)
action = tell | notify | tellchanged
subgrammar = Optional(cronspec) + patternmatch + Literal(':') + action

user = None
password = None
statement = ""

def dosub(sub, um):
	"""
	function to parse and execute a subscription statement
	takes one dictionary argument containing:
	user, password to be used in Access'ing the model and
	statement, a string containing the subscription statement
	"""
	global currentum
	currentum = um
	global defaultmodel
	defaultmodel = um.modelname
	user = sub['user']
	password = sub.get('password')
	statement = sub['statement']
	print "default model:", defaultmodel
	print "statement:", statement
	try:
		toks = subgrammar.parseString(statement)
	except Exception as err:
		print "parse failed [[%s]]" % (repr(err))
	else:
		return True



class Dummyum:
	def __init__(self, name):
		self.modelname = name
	
if __name__ == '__main__':
	testsub = "<froomBT/seen> ~ '.*' : TELL <froomBT/seen>/location, <froomBT/location>"
	testsub = "<froomBT/seen> ~ '.*' : NOTIFY 'http://www/qqq.cgi'"
	testsub = """
 <froomBT/seen> ~ '.*' :
	 TELL bob/personal/location, explicit:<froomBT/location>
"""
	testsub = """
 <bob/personal/location> ~ '.*' :
	 NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi'
"""
	testsub = """
 <default!bob/personal/location> ~ '.*' :
	 NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi'
"""
	testsub = """
 <default!bob/personal/location> ~ '.*' :
	 NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi'
"""
	testsub = """
<bobPhone/seenby> ~ '.*' : TELL bobPhone/location, explicit:<<bobPhone/seenby>/location>
"""
	testsub = """
 ["*/15 * * * *"] <default!./personal/location> ~ '.*' :
	 NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi?' 'location=' <./personal/location>  '&name=' <./personal/firstname>
"""
	testsub = """
 <default!./personal/location> ~ '.*' :
	 NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi?' 'location=' <./personal/location>  '&name=' <./personal/firstname>
"""
	print testsub
	dosub({'user':'bob', 'password':'qwert', 'statement':testsub}, Dummyum("bob"))




# The Personis system is copyright 2000-2012 University of Sydney
#       Bob.Kummerfeld@Sydney.edu.au

# This file is part of Personis.

# Personis is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Personis is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Personis.  If not, see <http://www.gnu.org/licenses/>.

from multiprocessing import Process, Queue
import time
from . import subscription
from . import base as pmdns
import logging

cronq = None # message q

crondb = {} # rule database
crondblen = 0

def time_in_spec(t, spec):
	return (t.tm_min in spec['minset']) and\
		(t.tm_hour in spec['hourset']) and\
		(t.tm_mday in spec['domset']) and\
		(t.tm_mon in spec['monthset']) and\
		(t.tm_wday in spec['dowset'])

def chkspec(t,spec,orig):
	full = dict(minset=set(range(60)), hourset = set(range(24)), domset = set(range(31)), monthset = set(range(12)), dowset = set(range(7)))
	now = dict(minset=set([t.tm_min]), hourset = set([t.tm_hour]), domset = set([t.tm_mday]), monthset = set([t.tm_mon]), dowset = set([t.tm_wday]))
	print "full = ",full
	print "spec = ",spec
	for s in ['minset','hourset','domset','monthset','dowset']:
		if spec[s] != full[s]:
			print "%s : %s" % (s, spec[s])
			if now[s].issubset(spec[s]):
				print ">>%s : %s" % (s, orig[s].difference(now[s]))
				speccopy = spec.copy()
				speccopy[s] = orig[s].difference(now[s])
				return speccopy
	return []

def checksub(modeldir, m):
	sub = m['subscription']
	try:	
		um = pmdns.Access(modeldir=modeldir, model=sub['modelname'], user=sub['user'], password=sub['password'])
	except:
		print "Access failed", sub
		return
	print "Access OK", sub
	try:
		subscription.dosub(sub,um)
	except:
		print "dosub failed"
		return
	print "dosub ok"
	

def cronserver(q, modeldir):
	global crondb, crondblen
	while True:
		try:
			m = q.get(True, 60) # block for 60 seconds at most
		except :
			m = None
		# check cron rules every 60 seconds at least
		# print "======", time.ctime()
		if m == None:
			for i in range(crondblen):
				Doco = '''
--------------------------------------------------------------------------------------------------------
initially newspec is []
newspec is either [] or cronspec minus the previous times if those times are immediately adjacent
eg if cronspec minutes were 3,4,6,7 then 
	at time 3 rule fires and newspec will become 4,6,7 
	at time 4 rule fires and newspec will become 3,6,7 
	at time 5 newspec will become 3,4,6,7
	at time 6 rule fires and newspec will become 3,4,7 
	at time 7 rule fires and newspec will become 3,4,6
	at time 8 newspec will become 3,4,6,7

if newspec is [] 
	if cronspec in current time
		fire rule
		set newspec to (cronspec - current time)
		
else
	if newspec in current time
		fire rule
		set newspec to (newspec - current time (for all parts))
		
		
--------------------------------------------------------------------------------------------------------
'''
				cronspec = crondb[i]['cronspec']
				newspec = crondb[i]['newspec']
				tnow = time.localtime()
				if newspec == []:
					if time_in_spec(tnow, cronspec):
						newspec = chkspec(tnow, cronspec, cronspec)
						print "=== FIRED cronspec ===", time.ctime(), newspec
						checksub(modeldir, crondb[i])
						#print i,":",crondb[i]
				else:
					if time_in_spec(tnow, newspec):
						newspec = chkspec(tnow, newspec, cronspec)
						print "=== FIRED newspec ===", time.ctime(), newspec
						checksub(modeldir, crondb[i])
						#print i,":",crondb[i]
				crondb[i]['newspec'] = newspec
			continue
		if m["op"] == "quit":
			break
		elif m["op"] == "put":
			cspec = m['subscription']['statement']
			(cspec,therest) = cspec.split("]",1)
			cspec = cspec.strip()[1:].strip().strip(''''"''')
			(min, hour, dom, month, dow) = cspec.split(" ", 4)
			#print "cspec = <%s>" % (cspec)
			#print "min = %s, hour = %s, dom = %s, month = %s, dow = %s" % (min,hour,dom,month,dow)
			minset = crontab_parser(60).parse(min)
			hourset = crontab_parser(24).parse(hour)
			domset = crontab_parser(31).parse(dom)
			monthset = crontab_parser(12).parse(month)
			dowset = crontab_parser(7).parse(dow)
			#print "minset = %s, hourset = %s, domset = %s, monthset = %s, dowset = %s" % (minset,hourset,domset,monthset,dowset)
			m['cronspec'] = dict(minset=minset, hourset=hourset, domset=domset, monthset=monthset, dowset=dowset)
			m['newspec'] = []
			crondb[crondblen] = m
			print "put:", m
			crondblen += 1


def weekday(dayname):
	return ["sun", "mon", "tue", "wed", "thu", "fri", "sat"].index(dayname[0:3].tolower())

class crontab_parser(object):
    """Parser for crontab expressions. Any expression of the form 'groups'
    (see BNF grammar below) is accepted and expanded to a set of numbers.
    These numbers represent the units of time that the crontab needs to
    run on::
 
        digit   :: '0'..'9'
        dow     :: 'a'..'z'
        number  :: digit+ | dow+
        steps   :: number
        range   :: number ( '-' number ) ?
        numspec :: '*' | range
        expr    :: numspec ( '/' steps ) ?
        groups  :: expr ( ',' expr ) *
 
    The parser is a general purpose one, useful for parsing hours, minutes and
    day_of_week expressions.  Example usage::
 
        >>> minutes = crontab_parser(60).parse("*/15")
        [0, 15, 30, 45]
        >>> hours = crontab_parser(24).parse("*/4")
        [0, 4, 8, 12, 16, 20]
        >>> day_of_week = crontab_parser(7).parse("*")
        [0, 1, 2, 3, 4, 5, 6]
 
    """
    ParseException = "ParseException"
 
    _range = r'(\w+?)-(\w+)'
    _steps = r'/(\w+)?'
    _star = r'\*'
 
    def __init__(self, max_=60):
	import re
        self.max_ = max_
        self.pats = (
                (re.compile(self._range + self._steps), self._range_steps),
                (re.compile(self._range), self._expand_range),
                (re.compile(self._star + self._steps), self._star_steps),
                (re.compile('^' + self._star + '$'), self._expand_star))
 
    def parse(self, spec):
        acc = set()
        for part in spec.split(','):
            if not part:
                raise self.ParseException("empty part")
            acc |= set(self._parse_part(part))
        return acc
 
    def _parse_part(self, part):
        for regex, handler in self.pats:
            m = regex.match(part)
            if m:
                return handler(m.groups())
        return self._expand_range((part, ))
 
    def _expand_range(self, toks):
        fr = self._expand_number(toks[0])
        if len(toks) > 1:
            to = self._expand_number(toks[1])
            return list(range(fr, min(to + 1, self.max_ + 1)))
        return [fr]
 
    def _range_steps(self, toks):
        if len(toks) != 3 or not toks[2]:
            raise self.ParseException("empty filter")
        return self._filter_steps(self._expand_range(toks[:2]), int(toks[2]))
 
    def _star_steps(self, toks):
        if not toks or not toks[0]:
            raise self.ParseException("empty filter")
        return self._filter_steps(self._expand_star(), int(toks[0]))
 
    def _filter_steps(self, numbers, steps):
        return [n for n in numbers if n % steps == 0]
 
    def _expand_star(self, *args):
        return list(range(self.max_))
 
    def _expand_number(self, s):
        if isinstance(s, basestring) and s[0] == '-':
            raise self.ParseException("negative numbers not supported")
        try:
            i = int(s)
        except ValueError:
            try:
                i = weekday(s)
            except KeyError:
                raise ValueError("Invalid weekday literal '%s'." % s)
        return i




#!/usr/bin/env python


# The Personis system is copyright 2000-2012 University of Sydney
#       Bob.Kummerfeld@Sydney.edu.au

# This file is part of Personis.

# Personis is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Personis is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Personis.  If not, see <http://www.gnu.org/licenses/>.

#
# Active User Models: added subscribe method to Access
#

import os, traceback
import json
import yaml
from . import jsoncall
import cherrypy
from . import base
from . import active
from . import util
from multiprocessing import Process, Queue
from . import cronserver
import random, time
#from ..client import Connection
from . import filedict
import string

from .mkmodel import *
import httplib2
import logging
import shutil
import threading

from oauth2client.file import Storage
from oauth2client.client import Storage, Credentials, OAuth2WebServerFlow, flow_from_clientsecrets

from genshi.template import TemplateLoader

class Server:
    '''The personis server

    :param modeldir: Directory containing the user models
    :type modeldir: str
    :param adminsfile: Path to list of people who can administrate the server
    :type adminsfile: str
    :param clients: Path to son file of oauth client details
    :type clients: str
    :param access_tokens: Path of file containint the oauth access tokens
    :type access_tokens: str
    :param client_secrets: Path to json file containing client info for this server (as a client) to access google APIs
    :type client_secrets: str
    '''

    def __init__(self, modeldir='models', adminsfile='admins.yaml', clients = None, access_tokens='./access_tokens.dat', client_secrets='client_secrets_google.json'):
        self.modeldir = modeldir
        self.admins = yaml.load(file(adminsfile,'r'))
        self.oauth_clients_file = clients
        self.client_secrets = client_secrets
        self.oauth_clients = json.loads(file(clients,'r').read())
        if self.oauth_clients == None:
            self.oauth_clients = {}
        self.access_tokens_filename = access_tokens
        def stopper():
            logging.info( 'saving persistant data')
            self.save_oauth_clients()
            #self.access_tokens.close()
        cherrypy.engine.subscribe('stop', stopper)

    def save_oauth_clients(self):
        f = open(self.oauth_clients_file,'w')
        s = json.dumps(self.oauth_clients)
        f.write(s)
        f.close()

    @cherrypy.expose
    def list_clients(self):
        # if we're here, we just want the local web UI. 
        # if no user then not logged in, so set admin to
        # true, target url back here, and go to login.
        if cherrypy.session.get('user') == None:
            cherrypy.session['admin'] = True
            cherrypy.session['target_url'] = '/list_clients'
            raise cherrypy.HTTPRedirect('/login')
        # check if user is actually allowed to be here
        if not cherrypy.session.get('user') in self.admins:
            raise cherrypy.HTTPError(401, 'Admin only')

        base_path = os.path.dirname(os.path.abspath(__file__))
        loader = TemplateLoader([base_path])
        tmpl = loader.load('html/list_clients.html')
        for k, v in self.oauth_clients.items():
            logging.debug( 'clients %s, %s', k, v['friendly_name'])
        stream = tmpl.generate(clients=self.oauth_clients.values())
        return stream.render('xhtml')


    @cherrypy.expose
    def list_apps(self):
        # if we're here, we just want the local web UI. 
        # if no user then not logged in, so set admin to
        # true, target url back here, and go to login.
        if cherrypy.session.get('user') == None:
            cherrypy.session['admin'] = True
            cherrypy.session['target_url'] = '/list_apps'
            raise cherrypy.HTTPRedirect('/login')

        base_path = os.path.dirname(os.path.abspath(__file__))
        loader = TemplateLoader([base_path])
        tmpl = loader.load('html/list_apps.html')
        um = cherrypy.session.get('um')
        apps = um.listapps()
        for k in apps:
            c = self.oauth_clients[k]
            apps[k]['friendly_name'] = c['friendly_name']
            apps[k]['client_id'] = c['client_id']
            apps[k]['icon'] = c['icon']
        apps = [apps[k] for k in apps]
        logging.debug(  'apps %s', apps)
        stream = tmpl.generate(apps = apps)
        return stream.render('xhtml')

    @cherrypy.expose
    def list_apps_save(self, id, value, _method='get'):
        if cherrypy.session.get('user') == None:
            raise cherrypy.HTTPError()
        # This uses a get parameter, where it should be del or post. 
        # worksfornow
        um = cherrypy.session.get('um')
        if id == "removeOneForMe":
            logging.debug(  "removed an app")
            um.deleteapp(value)
            raise cherrypy.HTTPRedirect('/list_apps')

    @cherrypy.expose
    def list_clients_save(self, id, value, _method='get'):
        if cherrypy.session.get('user') == None:
            raise cherrypy.HTTPError()
        if not cherrypy.session.get('user') in self.admins:
            raise cherrypy.HTTPError()
        # This uses a get parameter, where it should be del or post. 
        # worksfornow
        if id == "removeOneForMe":
            del(self.oauth_clients[value])
            self.save_oauth_clients()
            logging.debug(  "removed a client")
            raise cherrypy.HTTPRedirect('/list_clients')
        if id == "addOneForMe":
            clid = ''
            secret = ''
            for i in range(32):
                clid = clid + random.choice(string.hexdigits)
                secret = secret + random.choice(string.hexdigits)
            self.oauth_clients[clid] = {
                             'client_id': clid,
                             'friendly_name': 'my client',
                             'secret': secret, 
                             'redirect_uri': 'http://www.example.com/',
                             'icon': '/static/images/icon.svg'}
            self.save_oauth_clients()
            logging.debug(  "added a client")
            raise cherrypy.HTTPRedirect('/list_clients')

        clid, field = id.split('|')
        logging.debug(  'saving: %s, %s, %s',clid, field, value)
        oldc = self.oauth_clients[clid][field] = value
        for k, v in self.oauth_clients.items():
            logging.debug('%s, %s', k, v['friendly_name'])
        self.save_oauth_clients()
        return value
        
    @cherrypy.expose
    def authorize(self, client_id, redirect_uri, scope, access_type, response_type='code', approval_prompt='auto', state=None):
        """
        This is the entry point for client authentication against Personis. 
        (accessed by a user with a web browser, redirected from a client).
        Personis becomes an oauth2 server at this point. The session
        now contains the client_id but not much else.
        Only for oauth use. Don't come in this way if you want 
        to use list_clients!!
        """
        cherrypy.session['admin'] = False
        cherrypy.session['client_id'] = client_id
        
        cli = self.oauth_clients[client_id]
        if state != None:
            cherrypy.session['state'] = state
        if cli['redirect_uri'] != redirect_uri:
            logging.debug("redirect uris don't match expected %s got %s",cli['redirect_uri'], redirect_uri)
            raise cherrypy.HTTPError(400,'Redirect URIs do not match') 
        raise cherrypy.HTTPRedirect('/login')

    @cherrypy.expose
    def login(self):
        """
        Step 1.5. This is where clients of the personis web interface
        enter. there is no client_id etc because personis is not being
        used as an oauth server.
        """
        callback = cherrypy.request.base + '/logged_in'
        flow = flow_from_clientsecrets(
            self.client_secrets,
            scope='https://www.googleapis.com/auth/userinfo.profile https://www.googleapis.com/auth/userinfo.email',
            redirect_uri = callback)
        authorize_url = flow.step1_get_authorize_url()
        cherrypy.session['flow'] = flow
        raise cherrypy.HTTPRedirect(authorize_url)

    @cherrypy.expose
    def logged_in(self, code):
        """
        Step 2 of the oauth dance. At this point Google has said this is a 
        known user. We then go and get more user info about them from google,
        save the info in a persistant store, create a model if none exists,
        and then ask them if they're happy to let the client they're using 
        access personis on their behalf.
        (Accessed by the user with a web browser redirected from /authorized)
        """
        flow = cherrypy.session.get('flow')
        if not flow:
            raise IOError()
        credentials = flow.step2_exchange(
            cherrypy.request.params
            #, http = httplib2.Http(
            #    proxy_info = httplib2.ProxyInfo(proxy_type=httplib2.socks.PROXY_TYPE_HTTP_NO_TUNNEL, proxy_host='www-cache.it.usyd.edu.au', proxy_port=8000)
            #    )
        )
        http = httplib2.Http()
        http = credentials.authorize(http)
        cjson = credentials.to_json()
        cjson = json.loads(cjson)
        content = http.request('https://www.googleapis.com/oauth2/v1/userinfo?access_token='+cjson['access_token'])
        #print 'content', content
        try:
            usr = json.loads(content[1])
            user = usr['email'].split('@')[0].replace('.','')
            cherrypy.session['user'] = user
        except:
            logging.debug(  'exception on usr %s', content[1])
            raise IOError()


        logging.debug(  'loggedin session id %s',cherrypy.session.id)

        if not 'picture' in usr:
            if usr['gender'].lower() == 'male':
                usr['picture'] = 'http://upload.wikimedia.org/wikipedia/commons/thumb/e/ec/Mona_Lisa%2C_by_Leonardo_da_Vinci%2C_from_C2RMF_retouched.jpg/161px-Mona_Lisa%2C_by_Leonardo_da_Vinci%2C_from_C2RMF_retouched.jpg'
            else:
                usr['picture'] = 'http://www.lacasadeviena.com/wp-content/uploads/2012/06/magritte-sonofman1-300x362.jpg'

        # if no model for user, create one.
        if not os.path.exists(os.path.join(self.modeldir,user)):
            mf = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'modeldefs/user.prod.mdef')
            mkmodel(model=user, mfile=mf, modeldir=self.modeldir, user=user)
            um = active.Access(model=user, modeldir=self.modeldir, user=user)
            ev = base.Evidence(source="Create_Model", evidence_type="explicit", value=usr['given_name'])
            um.tell(context=["Personal"], componentid='firstname', evidence=ev)
            ev = base.Evidence(source="Create_Model", evidence_type="explicit", value=usr['family_name'])
            um.tell(context=["Personal"], componentid='lastname', evidence=ev)
            ev = base.Evidence(source="Create_Model", evidence_type="explicit", value=usr['gender'])
            um.tell(context=["Personal"], componentid='gender', evidence=ev)
            ev = base.Evidence(source="Create_Model", evidence_type="explicit", value=usr['email'])
            um.tell(context=["Personal"], componentid='email', evidence=ev)
            ev = base.Evidence(source="Create_Model", evidence_type="explicit", value=usr['id'])
            um.tell(context=["Personal"], componentid='gid', evidence=ev)
            ev = base.Evidence(source="Create_Model", evidence_type="explicit", value=usr['picture'])
            um.tell(context=["Personal"], componentid='picture', evidence=ev)


            reslist = um.ask(context=["Personal"], view=['firstname','email'])
            util.printcomplist(reslist)

        um = active.Access(model=user, modeldir=self.modeldir, user=user)
        
        # if we're here from a local url, just redirect. no need to allow.
        if cherrypy.session.get('admin'):
            cherrypy.session['um'] = um
            raise cherrypy.HTTPRedirect(cherrypy.session['target_url'])

        # if the app is already registered, it's ok.
        apps = um.listapps()
        logging.debug(  'apps: %s, clientid: %s', apps, cherrypy.session.get('client_id'))
        if cherrypy.session.get('client_id') in apps:
            raise cherrypy.HTTPRedirect('/allow')

        #otherwise, ask yes/no                
        cli = self.oauth_clients[cherrypy.session['client_id']]
        base_path = os.path.dirname(os.path.abspath(__file__))
        loader = TemplateLoader([os.path.join(base_path, 'html')])
        tmpl = loader.load('appQuery.html')
        stream = tmpl.generate(name=usr['given_name'], app=cli['friendly_name'], icon=cli['icon'], picture=usr['picture'])
        return stream.render('xhtml')

    @cherrypy.expose
    def allow(self):
        """
        In the case that they do want the client to use personis on their
        behalf, we register the client in um, and redirect back to the
        client with a temporary authorization key.
        (Accessed by a user with a web browser, redirected from /logged_in)
        """
        usrid = cherrypy.session.get('user')
        cli = self.oauth_clients[cherrypy.session.get('client_id')]

        val_key = ''
        for i in range(32):
            val_key = val_key+ random.choice(string.hexdigits)
        rdi = cli['redirect_uri'] + '?'
        if 'state' in cherrypy.session:
            rdi = rdi + 'state='+cherrypy.session['state']+'&amp;'
        rdi = rdi + 'code=' + val_key

        access_tokens = filedict.FileDict(filename=self.access_tokens_filename)

        access_tokens[val_key] = {'timestamp': time.time(), 'userid': usrid, 'client_id': cherrypy.session['client_id'], 'type': 'authorization_code', 'expires': time.time()+600}
        #self.access_tokens.sync()
        logging.info('access tokens: %s',repr([i for i in access_tokens.keys()]))
        
        redr = cli['redirect_uri']
        um = active.Access(model=usrid, modeldir=self.modeldir, user=usrid)
        cherrypy.session['um'] = um
        result = um.registerapp(app=cherrypy.session['client_id'], desc=cli['friendly_name'], password='')
        raise cherrypy.HTTPRedirect(rdi)
               
    @cherrypy.expose
    def dissallow(self):
        cli = self.oauth_clients[cherrypy.session.get('client_id')]
        redr = cli.redirect_uri+'?error=access_denied'
        #print redr
        raise cherrypy.HTTPRedirect(redr)

    @cherrypy.expose
    def request_token(self, client_id, client_secret, grant_type, code = None, redirect_uri = None, scope = None, refresh_token = None):
        """
        The client (mneme) has a temporary key (see /allow) but the key 
        has been to web browsers and back so it is not safe. It must be 
        exchanged by the client taking to personis directly (no web browser
        involvement) for a real token. Tokens have expiration dates etc.
        (Accessed by the client (Mneme, etc) on behalf of a user.)
        NOTE! This should only be exported over TLS/SSL (ahem!)
        """
        logging.debug(  'code %s, redirui %s, clientid %s, clientsec %s, scope %s, granttype %s, refreshtoken %s'%(code, redirect_uri, client_id, client_secret, scope, grant_type, refresh_token))
        cli = self.oauth_clients[client_id]

        # expire old tokens before we look
        now = time.time()

        access_tokens = filedict.FileDict(filename=self.access_tokens_filename)

        for k, v in access_tokens.items():
            logging.info(  'access_tokens %s: %s', k,v)
            if now > v['expires']:
                logging.info (  'expire access_token %s',k)
                del(access_tokens[k])

        if grant_type == 'refresh_token':
            code = refresh_token

        if not code in access_tokens:
            if grant_type == 'refresh_token':
                raise cherrypy.HTTPError(401, 'Refresh_token expiry')
            else:
                raise cherrypy.HTTPError(401, 'Incorrect token')

        tok = access_tokens[code]

        if tok['client_id'] != client_id:
            raise cherrypy.HTTPError(401, 'Incorrect client')
        
        if cli['secret'] != client_secret:
            raise cherrypy.HTTPError(401, 'Incorrect client information')

        userid = tok['userid']

        access_expiry =  3600 # an hour
        refresh_expiry = 3600*24*7*52 *20 # a year

        access_token = ''
        access_token = ''.join([random.choice(string.hexdigits) for i in range(32)])

        access_tokens[access_token] = {'timestamp': time.time(), 'userid': userid, 'client_id': client_id, 'type': 'access_token', 'expires': time.time() + access_expiry}
        logging.debug(  'added access_token: %s',access_token)

        ret = {'access_token': access_token, 
               'token_type': 'bearer',
               'expires_in': access_expiry,
               "example_parameter":"example_value"
               }

        if grant_type == 'authorization_code':
            refresh_token = ''
            refresh_token = ''.join([random.choice(string.hexdigits) for i in range(32)])
            logging.info(  'added refresh_token: %s',refresh_token)

            access_tokens[refresh_token] = {'timestamp': time.time(), 'userid': userid, 'client_id': client_id, 'type': 'refresh_token', 'expires': time.time() + refresh_expiry}

            ret['refresh_token'] = refresh_token

        #self.access_tokens.sync()

        s = json.dumps(ret)
        logging.info(  s)
        return s


    @cherrypy.expose
    def default(self, *args, **kargs):

        cherrypy.session['admin'] = False

        access_tokens = filedict.FileDict(filename=self.access_tokens_filename)

        try:
            access_token = cherrypy.request.headers['Authorization'].split()[1]
            logging.debug( 'access_tokens %s', access_tokens )
            logging.debug(  'access_token %s', repr(access_token) )
        except:
            return '''
<h1>Personis</h1>
Looks like you're coming into the service entrance with a browser, which is not how this works. Ask someme about 'Mneme', or perhaps you're looking for <a href="/list_apps">your apps</a>.
<br/>If you're an administrator, you might want to try <a href="/list_clients">the admin page</a>, or the <a href="/doc/index.html">the docs</a>.
<br/>Source at <a href="https://github.com/jbu/personis">github</a>.
'''
        if access_token not in access_tokens:
    	    raise cherrypy.HTTPError(401, 'Incorrect access token')
            #print 'token',self.access_tokens[access_token]
        now = time.time()
        if now > access_tokens[access_token]['expires']:
            logging.debug(  'expired %s', access_token)
            raise cherrypy.HTTPError(401, 'Expired access token')
    
        usr = access_tokens[access_token]['userid']

        cl = cherrypy.request.headers['Content-Length']
        jsonobj = cherrypy.request.body.read(int(cl))
        
        try:
            pargs = json.loads(jsonobj)
        except:
            logging.debug(  "bad request - cannot decode json - possible access from web browser")
            return json.dumps("Personis User Model server. Not accessible using a web browser.")

        # dirty kludge to get around unicode
        for k,v in pargs.items():
            if type(v) == type(''):
                pargs[k] = str(v)
            if type(k) == type(''):
                del pargs[k]
                pargs[str(k)] = v

        model = usr
        if 'model' in pargs:
            model = pargs['modelname']

        logging.debug(  'USER: %s, MODEL: %s, BEARER: %s', usr, model, access_token)

        try:
            result = False
            if args[0] == 'mkmodel':
                if not os.path.exists(os.path.join(self.modeldir,model)):
                    mkmodel(model=model, mfile='modeldefs/empty.prod.mdef', modeldir=self.modeldir, user=usr['id'], description=pargs['description'])
                result = True
            else:
                um = active.Access(model=model, modeldir=self.modeldir, user=usr)

            apps = um.listapps()
            if access_tokens[access_token]['client_id'] not in apps:
                logging.debug(  'client for access token not in model %s, %s', access_token, access_tokens[access_token]['client_id'])
                raise cherrypy.HTTPError(401, 'client for access token not in model')

            if args[0] == 'access':
                result = True
            elif args[0] == 'tell':
                result = um.tell(context=pargs['context'], componentid=pargs['componentid'], evidence=base.Evidence(**pargs['evidence']))
            elif args[0] == 'ask':
                reslist = um.ask(context=pargs['context'], view=pargs['view'], resolver=pargs['resolver'], \
                                        showcontexts=pargs['showcontexts'])
                if pargs['showcontexts']:
                    cobjlist, contexts, theviews, thesubs = reslist
                    cobjlist = [c.__dict__ for c in cobjlist]
                    for c in cobjlist:
                        if c["evidencelist"]:
                            c["evidencelist"] = [e for e in c["evidencelist"]]
                    newviews = {}
                    if theviews != None:
                        for vname,v in theviews.items():
                            newviews[vname] = v.__dict__
                    else:
                        newviews = None
                    reslist = [cobjlist, contexts, newviews, thesubs]
                else:
                    reslist = [c.__dict__ for c in reslist]
                    for c in reslist:
                        if c["evidencelist"]:
                            c["evidencelist"] = [e for e in c["evidencelist"]]
                result = reslist

            elif args[0] == 'subscribe':
                result = um.subscribe(context=pargs['context'], view=pargs['view'], subscription=pargs['subscription'])
            elif args[0] == 'delete_sub':
                result = um.delete_sub(context=pargs['context'], componentid=pargs['componentid'], subname=pargs['subname'])
            elif args[0] == 'list_subs':
                result = um.list_subs(context=pargs['context'], componentid=pargs['componentid'])
            elif args[0] == 'export_model':
                result = um.export_model(context=pargs['context'], resolver=pargs['resolver'])
            elif args[0] == 'import_model':
                result = um.import_model(context=pargs['context'], partial_model=pargs['partial_model'])
            elif args[0] == 'set_goals':
                result = um.set_goals(context=pargs['context'], componentid=pargs['componentid'], goals=pargs['goals'])
            elif args[0] == 'registerapp':
                result = um.registerapp(app=pargs['app'], desc=pargs['description'], password=pargs['apppassword'])
            elif args[0] == 'deleteapp':
                result = um.deleteapp(app=pargs['app'])
            elif args[0] == 'getpermission':
                result = um.getpermission(context=pargs['context'], componentid=pargs['componentid'], app=pargs['app'])
            elif args[0] == 'setpermission':
                result = um.setpermission(context=pargs['context'], componentid=pargs['componentid'], app=pargs['app'], permissions=pargs['permissions'])
            elif args[0] == 'listapps':
                result = um.listapps()
            elif args[0] == 'mkcomponent':
                comp = base.Component(**pargs["componentobj"])
                result = um.mkcomponent(pargs["context"], comp)
            elif args[0] == 'delcomponent':
                result = um.delcomponent(pargs["context"], pargs["componentid"])
            elif args[0] == 'delcontext':
                result = um.delcontext(pargs["context"])
            elif args[0] == 'setresolver':
                result = um.setresolver(pargs["context"], pargs["componentid"], pargs["resolver"])
            elif args[0] == 'getresolvers':
                result = um.getresolvers()
            elif args[0] == 'mkview':
                viewobj = base.View(**pargs["viewobj"])
                result = um.mkview(pargs["context"], viewobj)
            elif args[0] == 'delview':
                result = um.delview(pargs["context"], pargs["viewid"])
            elif args[0] == 'mkcontext':
                contextobj = base.Context(**pargs["contextobj"])
                result = um.mkcontext(pargs["context"], contextobj)
            elif args[0] == 'getcontext':
                result = um.getcontext(pargs["context"], pargs["getsize"])


            # Repackage result code with error values IF there is a version string.
            if 'version' in pargs:
                new_result = {}
                new_result["result"] = "ok"
                new_result["val"] = result
                result = new_result

        except Exception as e:

            logging.info(  "Exception: %s", e)
            traceback.print_exc()
            if 'version' in pargs:
                new_result = {}
                new_result["val"] = [e.__class__.__name__, e.__dict__.copy(), cPickle.dumps(e)]
                new_result["result"] = "error"
                result = new_result
            else:
                result = False

        return json.dumps(result)

class ExitThread(threading.Thread):
    def __init__(self, cp, cronp, cronqueue, queue):
        self.cp = cp
        self.q = queue
        self.cronq = cronqueue
        self.cronp = cronp

    def run(self):
        self.running = True
        print 'exit thread running'
        logging.info('exit listener running')
        while self.running:
            g = self.q.get(block = True)
            logging.info('exit thread got %s', g)
            if g == 'exit':
                self.running = False
            print 'qgot', g
        logging.info('exit listener run ends')
        self.exit()

    def exit(self):
        logging.info(  "Shutting down Personis Server. - exit thread")
        self.cronq.put(dict(op="quit"))
        self.cronp.join()
        self.cp.engine.exit()
        self.join()

def runServer(modeldir, config, admins, clients, tokens, loglevel=logging.INFO, exit_queue = None, client_secrets = 'client_secrets_google.json'):
    '''Utility to run the personis server.

    :param modeldir: Directory containing the user models
    :type modeldir: str
    :param config: Path to cherrypy server config file
    :type config: str
    :param admins: Path to list of people who can administrate the server
    :type admins: str
    :param clients: Path to son file of oauth client details
    :type clients: str
    :param tokens: Path of file containint the oauth access tokens
    :type tokens: str
    :param client_secrets: Path to json file containing client info for this server (as a client) to access google APIs
    :type client_secrets: str
    '''
    logging.basicConfig(level=loglevel)
    logging.info(  "serving models in '%s'" % (modeldir))
    logging.info(  "config file '%s'" % (config))
    logging.info(  "admin file '%s'" % (admins))
    logging.info(  "clients file '%s'" % (clients))
    logging.info(  "tokens file '%s'" % (tokens))
    logging.info(  "client secrets file '%s'" % (client_secrets))
    logging.info(  "starting cronserver")
    cronserver.cronq = Queue()
    p = Process(target=cronserver.cronserver, args=(cronserver.cronq,modeldir))
    p.start()

    if exit_queue:
        e = ExitThread(cherrypy, p, cronserver.cronq, exit_queue)

    try:
        try:
            cherrypy.config.update(os.path.expanduser(config))
            cherrypy.tree.mount(Server(modeldir, admins, clients, tokens, client_secrets), '/', config=config)
            #cherrypy.server.ssl_certificate = "server.crt"
            #cherrypy.server.ssl_private_key = "server.key" 
            cherrypy.engine.start()
            if not exit_queue:
                cherrypy.engine.block()
            else:
                e.start()
        except Exception as E:
            logging.info(  "Failed to run Personis Server:" + str(E))
    finally:
        if exit_queue:
            exit_queue.put('exit')
        else:
            logging.info(  "Shutting down Personis Server.")
            cronserver.cronq.put(dict(op="quit"))
            p.join()




#!/usr/bin/env python

import Personis_base
from Personis_util import printcomplist, showobj

# create a new context
um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')

print("creating context 'Deltest' in context 'Personal'")
cobj = Personis_base.Context(Identifier="Deltest", Description="testing context deletion")
um.mkcontext(context=["Personal"], contextobj=cobj)

print("getting context information:")
print("Info with size:", um.getcontext(context=["Personal"], getsize=True))
print(um.getcontext(context=["Personal", "Deltest"]))

print("now deleting it")
if um.delcontext(context=["Personal", "Deltest"]):
	print("ok")
else:
	print("fail")

print("check for tgz file in Models directory")



#!/usr/bin/env python

import Personis_base
from Personis_util import showobj, printcomplist

def printAskContext( info ):
	(cobjlist, contexts, theviews, thesubs) = info
	printcomplist(cobjlist, printev = "yes")
	print("Contexts: %s" % str(contexts))
	print("Views: %s" % str(theviews))
	print("Subscriptions: %s" % str(thesubs))
		
print("===================================================================")
print("Examples that show how the showcontext call works")
print("===================================================================")

um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')

print("1. Show the root context")
info = um.ask(context=[""], showcontexts=True)
printAskContext (info)

print("2. Show the Personal context")
info = um.ask(context=["Personal"], showcontexts=True)
printAskContext (info)



#!/usr/local/bin/python

import Personis
import Personis_a

print(">>>> subscribe to changes in lastname")

um = Personis.Access(model='Alice', user='alice', password='secret')

sub = """
<default!./Personal/lastname> ~ '.*' :
         NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi?' 'lastname=' <./Personal/lastname> 
"""

result = um.subscribe(context=["Personal"], view=['lastname'], subscription={'user':'alice', 'password':'qwert', 'statement':sub})
print(result)

print(">>>> add a subscription that is checked regularly according to a cron rule")
# min, hour, day of month, month, day of week
sub = """
 ["1-58 * * * *"] <default!./Personal/lastname> ~ '.*' :
         NOTIFY 'http://www.it.usyd.edu.au/~bob/Personis/tst.cgi?' 'lastname=' <./Personal/lastname> 
"""

result = um.subscribe(context=["Personal"], view=['lastname'], subscription={'user':'alice', 'password':'secret', 'statement':sub})
print(result)



#!/usr/bin/env python

from optparse import OptionParser
import logging
import os
from .server import runServer

parser = OptionParser()
parser.add_option("-m", "--models", dest="modeldir",
          help="Model directory", metavar="DIRECTORY", default='models')
parser.add_option("-c", "--config",
          dest="conf", metavar='FILE', default="server.conf",
          help="Config file")
parser.add_option("-a", "--admins",
          dest="admins", metavar='FILE',
          help="Admins file", default='admins.yaml')
parser.add_option("-o", "--oauthclients",
          dest="clients", metavar='FILE',
          help="Clients json file", default='oauth_clients.json')
parser.add_option("-t", "--tokens",
          dest="tokens", metavar='FILE',
          help="Access tokens database", default='access_tokens.dat')
parser.add_option("-l", "--log",
          dest="logging",
          help="Log level", default='INFO')
parser.add_option("-s", "--clientsecrets",
          dest="client_secrets",
          help="Client secrets file", default='client_secrets_google.json')

(options, args) = parser.parse_args()

numeric_level = getattr(logging, options.logging.upper(), None)
#cherrypy.log('Debugging%s, %d', options.logging, numeric_level)
if not isinstance(numeric_level, int):
    raise ValueError('Invalid log level: %s' % options.logging)
logging.basicConfig(level=numeric_level)

options.modeldir = os.path.abspath(options.modeldir)
options.conf = os.path.abspath(options.conf)
options.admins = os.path.abspath(options.admins)
options.clients = os.path.abspath(options.clients)
options.tokens = os.path.abspath(options.tokens)
options.client_secrets = os.path.abspath(options.client_secrets)

runServer(modeldir=options.modeldir, config=options.conf, admins=options.admins, clients=options.clients, tokens=options.tokens, loglevel=numeric_level, client_secrets=options.client_secrets)



#!/usr/bin/env python

import Personis_base
from Personis_util import printcomplist
import json


print("add a JSON encoded value to a component")
fullname = {'firstname':'Alice', 'lastname':'Smith'}
um = Personis_base.Access(model="Alice", modeldir='Tests/Models', user='alice', password='secret')
# create a piece of evidence with json encoded value
ev = Personis_base.Evidence(evidence_type="explicit", value=json.dumps(fullname))
um.tell(context=["People"], componentid='fullname', evidence=ev)

print("===================================================================")
print("Now check the evidence list ")
reslist = um.ask(context=["People"], view=['fullname'])
printcomplist(reslist, printev = "yes")

